@@ -10,6 +10,23 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+import copy
+import itertools
+
+import six
+
+from heat.common import exception
+from heat.common.i18n import _
+from heat.engine import attributes
+from heat.engine import constraints
+from heat.engine import function
+from heat.engine import properties
+from heat.engine.properties import Properties
+from heat.engine import resource
+from heat.engine import scheduler
+from heat.openstack.common import log as logging
+
 try:
     from pyrax.exceptions import NotFound
     PYRAX_INSTALLED = True
@@ -20,19 +37,7 @@ except ImportError:
 
     PYRAX_INSTALLED = False
 
-import copy
-import itertools
-
-from heat.openstack.common import log as logging
-from heat.openstack.common.gettextutils import _
-from heat.engine import scheduler
-from heat.engine import constraints
-from heat.engine import properties
-from heat.engine import resource
-from heat.engine.properties import Properties
-from heat.common import exception
-
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class LoadbalancerBuildError(exception.HeatException):
@@ -41,6 +46,8 @@ class LoadbalancerBuildError(exception.HeatException):
 
 class CloudLoadBalancer(resource.Resource):
 
+    """Represents a Rackspace Cloud Loadbalancer."""
+
     PROPERTIES = (
         NAME, NODES, PROTOCOL, ACCESS_LIST, HALF_CLOSED, ALGORITHM,
         CONNECTION_LOGGING, METADATA, PORT, TIMEOUT,
@@ -80,9 +87,9 @@ class CloudLoadBalancer(resource.Resource):
     )
 
     _VIRTUAL_IP_KEYS = (
-        VIRTUAL_IP_TYPE, VIRTUAL_IP_IP_VERSION,
+        VIRTUAL_IP_TYPE, VIRTUAL_IP_IP_VERSION, VIRTUAL_IP_ID
     ) = (
-        'type', 'ipVersion',
+        'type', 'ipVersion', 'id'
     )
 
     _HEALTH_MONITOR_KEYS = (
@@ -111,6 +118,15 @@ class CloudLoadBalancer(resource.Resource):
         'secureTrafficOnly',
     )
 
+    ATTRIBUTES = (
+        PUBLIC_IP, VIPS
+    ) = (
+        'PublicIp', 'virtualIps'
+    )
+
+    ALGORITHMS = ["LEAST_CONNECTIONS", "RANDOM", "ROUND_ROBIN",
+                  "WEIGHTED_LEAST_CONNECTIONS", "WEIGHTED_ROUND_ROBIN"]
+
     _health_monitor_schema = {
         HEALTH_MONITOR_ATTEMPTS_BEFORE_DEACTIVATION: properties.Schema(
             properties.Schema.NUMBER,
@@ -238,7 +254,10 @@ class CloudLoadBalancer(resource.Resource):
             properties.Schema.BOOLEAN
         ),
         ALGORITHM: properties.Schema(
-            properties.Schema.STRING
+            properties.Schema.STRING,
+            constraints=[
+                constraints.AllowedValues(ALGORITHMS)
+            ]
         ),
         CONNECTION_LOGGING: properties.Schema(
             properties.Schema.BOOLEAN
@@ -298,7 +317,9 @@ class CloudLoadBalancer(resource.Resource):
                 schema={
                     VIRTUAL_IP_TYPE: properties.Schema(
                         properties.Schema.STRING,
-                        required=True,
+                        "The type of VIP (public or internal). This property"
+                        " cannot be specified if 'id' is specified. This "
+                        "property must be specified if id is not specified.",
                         constraints=[
                             constraints.AllowedValues(['SERVICENET',
                                                        'PUBLIC']),
@@ -306,14 +327,25 @@ class CloudLoadBalancer(resource.Resource):
                     ),
                     VIRTUAL_IP_IP_VERSION: properties.Schema(
                         properties.Schema.STRING,
-                        default='IPV6',
+                        "IP version of the VIP. This property cannot be "
+                        "specified if 'id' is specified. This property must "
+                        "be specified if id is not specified.",
                         constraints=[
                             constraints.AllowedValues(['IPV6', 'IPV4']),
                         ]
                     ),
+                    VIRTUAL_IP_ID: properties.Schema(
+                        properties.Schema.NUMBER,
+                        "ID of a shared VIP to use instead of creating a "
+                        "new one. This property cannot be specified if type"
+                        " or version is specified."
+                    )
                 },
             ),
-            required=True
+            required=True,
+            constraints=[
+                constraints.Length(min=1)
+            ]
         ),
         CONTENT_CACHING: properties.Schema(
             properties.Schema.STRING,
@@ -358,27 +390,33 @@ class CloudLoadBalancer(resource.Resource):
     }
 
     attributes_schema = {
-        'PublicIp': _('Public IP address of the specified '
-                      'instance.')}
-
-    update_allowed_keys = ('Properties',)
+        PUBLIC_IP: attributes.Schema(
+            _('Public IP address of the specified instance.')
+        ),
+        VIPS: attributes.Schema(
+            _("A list of assigned virtual ip addresses")
+        )
+    }
 
     def __init__(self, name, json_snippet, stack):
         super(CloudLoadBalancer, self).__init__(name, json_snippet, stack)
         self.clb = self.cloud_lb()
 
     def cloud_lb(self):
-        return self.stack.clients.cloud_lb()
+        return self.client('cloud_lb')
 
     def _setup_properties(self, properties, function):
         """Use defined schema properties as kwargs for loadbalancer objects."""
         if properties and function:
-            return [function(**item_dict) for item_dict in properties]
+            return [function(**self._remove_none(item_dict))
+                    for item_dict in properties]
         elif function:
             return [function()]
 
     def _alter_properties_for_api(self):
-        """The following properties have usless key/value pairs which must
+        """Set up required, but useless, key/value pairs.
+
+        The following properties have useless key/value pairs which must
         be passed into the api. Set them up to make template definition easier.
         """
         session_persistence = None
@@ -392,7 +430,8 @@ class CloudLoadBalancer(resource.Resource):
         metadata = None
         if self.METADATA in self.properties.data:
             metadata = [{'key': k, 'value': v}
-                        for k, v in self.properties[self.METADATA].iteritems()]
+                        for k, v
+                        in six.iteritems(self.properties[self.METADATA])]
 
         return (session_persistence, connection_logging, metadata)
 
@@ -405,8 +444,9 @@ class CloudLoadBalancer(resource.Resource):
             return False
 
     def _configure_post_creation(self, loadbalancer):
-        """Configure all load balancer properties that must be done post
-        creation.
+        """Configure all load balancer properties post creation.
+
+        These properties can only be set after the load balancer is created.
         """
         if self.properties[self.ACCESS_LIST]:
             while not self._check_status(loadbalancer, ['ACTIVE']):
@@ -456,6 +496,7 @@ class CloudLoadBalancer(resource.Resource):
         node_list = self._process_nodes(self.properties.get(self.NODES))
         nodes = [self.clb.Node(**node) for node in node_list]
         vips = self.properties.get(self.VIRTUAL_IPS)
+
         virtual_ips = self._setup_properties(vips, self.clb.VirtualIP)
 
         (session_persistence, connection_logging, metadata) = \
@@ -479,7 +520,7 @@ class CloudLoadBalancer(resource.Resource):
 
         lb_name = (self.properties.get(self.NAME) or
                    self.physical_resource_name())
-        logger.debug(_("Creating loadbalancer: %s") % {lb_name: lb_body})
+        LOG.debug("Creating loadbalancer: %s" % {lb_name: lb_body})
         loadbalancer = self.clb.create(lb_name, **lb_body)
         self.resource_id_set(str(loadbalancer.id))
 
@@ -492,9 +533,7 @@ class CloudLoadBalancer(resource.Resource):
         return self._check_status(loadbalancer, ['ACTIVE'])
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        """
-        Add and remove nodes specified in the prop_diff.
-        """
+        """Add and remove nodes specified in the prop_diff."""
         loadbalancer = self.clb.get(self.resource_id)
         if self.NODES in prop_diff:
             current_nodes = loadbalancer.nodes
@@ -553,21 +592,18 @@ class CloudLoadBalancer(resource.Resource):
         else:
             if loadbalancer.status != 'DELETED':
                 loadbalancer.delete()
-                self.resource_id_set(None)
 
     def _remove_none(self, property_dict):
-        '''
-        Remove values that may be initialized to None and would cause problems
-        during schema validation.
-        '''
+        """Remove None values that would cause schema validation problems.
+
+        These are values that may be initialized to None.
+        """
         return dict((key, value)
-                    for (key, value) in property_dict.iteritems()
-                    if value)
+                    for (key, value) in six.iteritems(property_dict)
+                    if value is not None)
 
     def validate(self):
-        """
-        Validate any of the provided params
-        """
+        """Validate any of the provided params."""
         res = super(CloudLoadBalancer, self).validate()
         if res:
             return res
@@ -575,45 +611,64 @@ class CloudLoadBalancer(resource.Resource):
         if self.properties.get(self.HALF_CLOSED):
             if not (self.properties[self.PROTOCOL] == 'TCP' or
                     self.properties[self.PROTOCOL] == 'TCP_CLIENT_FIRST'):
-                return {'Error':
-                        'The %s property is only available for the TCP or '
-                        'TCP_CLIENT_FIRST protocols' % self.HALF_CLOSED}
+                message = (_('The %s property is only available for the TCP '
+                             'or TCP_CLIENT_FIRST protocols')
+                           % self.HALF_CLOSED)
+                raise exception.StackValidationFailed(message=message)
 
         #health_monitor connect and http types require completely different
         #schema
         if self.properties.get(self.HEALTH_MONITOR):
-            health_monitor = \
-                self._remove_none(self.properties[self.HEALTH_MONITOR])
+            prop_val = self.properties[self.HEALTH_MONITOR]
+            health_monitor = self._remove_none(prop_val)
 
             schema = self._health_monitor_schema
             if health_monitor[self.HEALTH_MONITOR_TYPE] == 'CONNECT':
                 schema = dict((k, v) for k, v in schema.items()
                               if k in self._HEALTH_MONITOR_CONNECT_KEYS)
-            try:
-                Properties(schema,
-                           health_monitor,
-                           self.stack.resolve_runtime_data,
-                           self.name).validate()
-            except exception.StackValidationFailed as svf:
-                return {'Error': str(svf)}
+            Properties(schema,
+                       health_monitor,
+                       function.resolve,
+                       self.name).validate()
 
-    def _public_ip(self):
-        #TODO(andrew-plunk) return list here and let caller choose ip
-        for ip in self.clb.get(self.resource_id).virtual_ips:
+        # if a vip specifies and id, it can't specify version or type;
+        # otherwise version and type are required
+        for vip in self.properties.get(self.VIRTUAL_IPS, []):
+            has_id = vip.get(self.VIRTUAL_IP_ID) is not None
+            has_version = vip.get(self.VIRTUAL_IP_IP_VERSION) is not None
+            has_type = vip.get(self.VIRTUAL_IP_TYPE) is not None
+            if has_id:
+                if (has_version or has_type):
+                    message = _("Cannot specify type or version if VIP id is"
+                                " specified.")
+                    raise exception.StackValidationFailed(message=message)
+            elif not (has_version and has_type):
+                message = _("Must specify VIP type and version if no id "
+                            "specified.")
+                raise exception.StackValidationFailed(message=message)
+
+    def _public_ip(self, lb):
+        for ip in lb.virtual_ips:
             if ip.type == 'PUBLIC':
-                return ip.address
+                return unicode(ip.address)
 
     def _resolve_attribute(self, key):
-        attribute_function = {
-            'PublicIp': self._public_ip()
-        }
-        if key not in attribute_function:
-            raise exception.InvalidTemplateAttribute(resource=self.name,
-                                                     key=key)
-        function = attribute_function[key]
-        logger.info(_('%(name)s.GetAtt(%(key)s) == %(function)s'),
-                    {'name': self.name, 'key': key, 'function': function})
-        return unicode(function)
+        if self.resource_id:
+            lb = self.clb.get(self.resource_id)
+            attribute_function = {
+                self.PUBLIC_IP: self._public_ip(lb),
+                self.VIPS: [{"id": vip.id,
+                             "type": vip.type,
+                             "ip_version": vip.ip_version}
+                            for vip in lb.virtual_ips]
+            }
+            if key not in attribute_function:
+                raise exception.InvalidTemplateAttribute(resource=self.name,
+                                                         key=key)
+            function = attribute_function[key]
+            LOG.info(_('%(name)s.GetAtt(%(key)s) == %(function)s'),
+                     {'name': self.name, 'key': key, 'function': function})
+            return function
 
 
 def resource_mapping():
