@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -14,13 +14,16 @@
 
 import copy
 import json
+import mock
+import six
 import uuid
 
-from heat.common.exception import StackValidationFailed
+from heat.common import exception
 from heat.common import template_format
 from heat.engine import resource
+from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import utils
 
 from ..resources import cloud_loadbalancer as lb  # noqa
@@ -51,7 +54,7 @@ class FakeManager(object):
     def find(self, *args, **kwargs):
         pass
 
-    def action(self, item, action_type, body={}):
+    def action(self, item, action_type, body=None):
         pass
 
 
@@ -152,7 +155,7 @@ def override_resource():
     }
 
 
-class LoadBalancerTest(HeatTestCase):
+class LoadBalancerTest(common.HeatTestCase):
 
     def setUp(self):
         super(LoadBalancerTest, self).setUp()
@@ -203,7 +206,6 @@ class LoadBalancerTest(HeatTestCase):
         }
 
         lb.resource_mapping = override_resource
-        utils.setup_dummy_db()
         resource._register_class("Rackspace::Cloud::LoadBalancer",
                                  LoadBalancerWithFakeClient)
 
@@ -216,9 +218,10 @@ class LoadBalancerTest(HeatTestCase):
                                  .__getattribute__(name))
         return super(LoadBalancerTest, self).__getattribute__(name)
 
-    def _mock_create(self, t, stack, resource_name, lb_name, lb_body):
+    def _mock_create(self, tmpl, stack, resource_name, lb_name, lb_body):
+        resource_defns = tmpl.resource_definitions(stack)
         rsrc = LoadBalancerWithFakeClient(resource_name,
-                                          t['Resources'][resource_name],
+                                          resource_defns[resource_name],
                                           stack)
         self.m.StubOutWithMock(rsrc.clb, 'create')
         fake_loadbalancer = FakeLoadBalancer(name=lb_name)
@@ -232,7 +235,7 @@ class LoadBalancerTest(HeatTestCase):
         t = template_format.parse(json.dumps(lb_template))
         s = utils.parse_stack(t, stack_name=utils.random_name())
 
-        rsrc, fake_loadbalancer = self._mock_create(t, s,
+        rsrc, fake_loadbalancer = self._mock_create(s.t, s,
                                                     self.
                                                     _get_first_resource_name(
                                                         lb_template),
@@ -246,13 +249,13 @@ class LoadBalancerTest(HeatTestCase):
         return (rsrc, fake_loadbalancer)
 
     def _set_template(self, templ, **kwargs):
-        for k, v in kwargs.iteritems():
+        for k, v in six.iteritems(kwargs):
             templ['Resources'][self._get_first_resource_name(templ)][
                 'Properties'][k] = v
         return templ
 
     def _set_expected(self, expected, **kwargs):
-        for k, v in kwargs.iteritems():
+        for k, v in six.iteritems(kwargs):
             expected[k] = v
         return expected
 
@@ -304,6 +307,34 @@ class LoadBalancerTest(HeatTestCase):
         scheduler.TaskRunner(rsrc.create)()
         self.m.VerifyAll()
 
+    def test_validate_vip(self):
+        snippet = {
+            "nodes": [],
+            "protocol": 'HTTP',
+            "port": 80,
+            "halfClosed": None,
+            "algorithm": u'LEAST_CONNECTIONS',
+            "virtualIps": [{"id": "1234"}]
+        }
+        stack = mock.Mock()
+        stack.db_resource_get.return_value = None
+        # happy path
+        resdef = rsrc_defn.ResourceDefinition("testvip",
+                                              lb.CloudLoadBalancer,
+                                              properties=snippet)
+        rsrc = lb.CloudLoadBalancer("testvip", resdef, stack)
+        self.assertIsNone(rsrc.validate())
+        # make sure the vip id prop is exclusive
+        snippet["virtualIps"][0]["type"] = "PUBLIC"
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                rsrc.validate)
+        self.assertIn("Cannot specify type or version", str(exc))
+        # make sure you have to specify type and version if no id
+        snippet["virtualIps"] = [{}]
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                rsrc.validate)
+        self.assertIn("Must specify VIP type and version", str(exc))
+
     def test_validate_half_closed(self):
         #test failure (invalid protocol)
         template = self._set_template(self.lb_template, halfClosed=True)
@@ -311,11 +342,10 @@ class LoadBalancerTest(HeatTestCase):
         rsrc, fake_loadbalancer = self._mock_loadbalancer(template,
                                                           self.lb_name,
                                                           expected)
-        self.assertEqual(
-            {'Error':
-             'The halfClosed property is only available for the '
-             'TCP or TCP_CLIENT_FIRST protocols'},
-            rsrc.validate())
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                rsrc.validate)
+        self.assertIn('The halfClosed property is only available for the TCP'
+                      ' or TCP_CLIENT_FIRST protocols', str(exc))
 
         #test TCP protocol
         template = self._set_template(template, protocol='TCP')
@@ -363,8 +393,9 @@ class LoadBalancerTest(HeatTestCase):
         rsrc, fake_loadbalancer = self._mock_loadbalancer(template,
                                                           self.lb_name,
                                                           expected)
-        self.assertEqual({'Error': 'Unknown Property bodyRegex'},
-                         rsrc.validate())
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                rsrc.validate)
+        self.assertIn('Unknown Property bodyRegex', str(exc))
 
         #test http fields
         health_monitor['type'] = 'HTTP'
@@ -398,8 +429,9 @@ class LoadBalancerTest(HeatTestCase):
                                                           self.lb_name,
                                                           expected)
 
-        exc = self.assertRaises(StackValidationFailed, rsrc.validate)
-        self.assertIn("Property certificate not assigned", str(exc))
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                rsrc.validate)
+        self.assertIn("Property certificate not assigned", six.text_type(exc))
 
         ssl_termination['certificate'] = 'dfaewfwef'
         template = self._set_template(template,
@@ -534,3 +566,10 @@ class LoadBalancerTest(HeatTestCase):
         self.m.ReplayAll()
         self.assertRaises(ValueError, rsrc.handle_update, {}, {}, deleted_node)
         self.m.VerifyAll()
+
+    def test_resolve_attr_noid(self):
+        stack = mock.Mock()
+        stack.db_resource_get.return_value = None
+        resdef = mock.Mock(spec=rsrc_defn.ResourceDefinition)
+        lbres = lb.CloudLoadBalancer("test", resdef, stack)
+        self.assertIsNone(lbres._resolve_attribute("PublicIp"))
