@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,16 +14,18 @@
 
 import itertools
 
+from docutils import core
+from docutils import nodes
+import pydoc
+from sphinx.util.compat import Directive
+
+from heat.common.i18n import _
+from heat.engine import attributes
 from heat.engine import environment
 from heat.engine import plugin_manager
-from heat.engine import resources
 from heat.engine import properties
+from heat.engine import resources
 from heat.engine import support
-from heat.openstack.common.gettextutils import _
-
-from docutils import nodes
-from sphinx.util.compat import Directive
-import pydoc
 
 
 global_env = environment.Environment({}, user_env=False)
@@ -51,18 +52,23 @@ class ResourcePages(Directive):
 
             self.props_schemata = properties.schemata(
                 self.resource_class.properties_schema)
+            self.attrs_schemata = attributes.schemata(
+                self.resource_class.attributes_schema)
 
             if resource_class.support_status.status == support.DEPRECATED:
                 sstatus = resource_class.support_status.to_dict()
-                para = nodes.inline(
-                    '', _('%(status)s - %(message)s') % sstatus)
+                msg = _('%(status)s')
+                if sstatus['message'] is not None:
+                    msg = _('%(status)s - %(message)s')
+                para = nodes.inline('', msg % sstatus)
                 warning = nodes.note('', para)
                 section.append(warning)
 
             cls_doc = pydoc.getdoc(resource_class)
             if cls_doc:
-                para = nodes.paragraph('', cls_doc)
-                section.append(para)
+                # allow for rst in the class comments
+                cls_nodes = core.publish_doctree(cls_doc).children
+                section.extend(cls_nodes)
 
             self.contribute_properties(section)
             self.contribute_attributes(section)
@@ -109,16 +115,19 @@ class ResourcePages(Directive):
                 props.append('%s: %s' % (prop_key,
                                          self._prop_syntax_example(prop)))
 
+        props_str = ''
+        if props:
+            props_str = '''\n    properties:
+      %s''' % ('\n      '.join(props))
+
         template = '''heat_template_version: 2013-05-23
 ...
 resources:
   ...
   the_resource:
-    type: %s
-    properties:
-      %s''' % (self.resource_type, '\n      '.join(props))
+    type: %s%s''' % (self.resource_type, props_str)
 
-        block = nodes.literal_block('', template)
+        block = nodes.literal_block('', template, language="hot")
         section.append(block)
 
     def contribute_yaml_syntax(self, parent):
@@ -131,16 +140,19 @@ resources:
                 props.append('%s: %s' % (prop_key,
                                          self._prop_syntax_example(prop)))
 
+        props_str = ''
+        if props:
+            props_str = '''\n    Properties:
+      %s''' % ('\n      '.join(props))
+
         template = '''HeatTemplateFormatVersion: '2012-12-12'
 ...
 Resources:
   ...
   TheResource:
-    Type: %s
-    Properties:
-      %s''' % (self.resource_type, '\n      '.join(props))
+    Type: %s%s''' % (self.resource_type, props_str)
 
-        block = nodes.literal_block('', template)
+        block = nodes.literal_block('', template, language='yaml')
         section.append(block)
 
     def contribute_json_syntax(self, parent):
@@ -153,19 +165,24 @@ Resources:
                     and prop.support_status.status == support.SUPPORTED):
                 props.append('"%s": %s' % (prop_key,
                                            self._prop_syntax_example(prop)))
+
+        props_str = ''
+        if props:
+            props_str = ''',\n      "Properties": {
+        %s
+      }''' % (',\n        '.join(props))
+
         template = '''{
   "AWSTemplateFormatVersion" : "2010-09-09",
   ...
   "Resources" : {
     "TheResource": {
-      "Type": "%s",
-      "Properties": {
-        %s
-      }
+      "Type": "%s"%s
     }
   }
-}''' % (self.resource_type, ',\n        '.join(props))
-        block = nodes.literal_block('', template)
+}''' % (self.resource_type, props_str)
+
+        block = nodes.literal_block('', template, language="json")
         section.append(block)
 
     @staticmethod
@@ -192,9 +209,11 @@ Resources:
         prop_item.append(definition)
 
         if prop.support_status.status != support.SUPPORTED:
-            para = nodes.inline(
-                '',
-                _('%(status)s - %(message)s') % prop.support_status.to_dict())
+            sstatus = prop.support_status.to_dict()
+            msg = _('%(status)s')
+            if sstatus['message'] is not None:
+                msg = _('%(status)s - %(message)s')
+            para = nodes.inline('', msg % sstatus)
             warning = nodes.note('', para)
             definition.append(warning)
 
@@ -212,6 +231,11 @@ Resources:
             para = nodes.paragraph('',
                                    _('Can be updated without replacement.'))
             definition.append(para)
+        elif prop.immutable:
+            para = nodes.paragraph('', _('Updates are not supported. '
+                                         'Resource update will fail on any '
+                                         'attempt to update this property.'))
+            definition.append(para)
         else:
             para = nodes.paragraph('', _('Updates cause replacement.'))
             definition.append(para)
@@ -232,13 +256,16 @@ Resources:
 
         sub_schema = None
         if prop.schema and prop.type == properties.Schema.MAP:
-            para = nodes.emphasis('', _('Map properties:'))
+            para = nodes.paragraph()
+            emph = nodes.emphasis('', _('Map properties:'))
+            para.append(emph)
             definition.append(para)
             sub_schema = prop.schema
 
         elif prop.schema and prop.type == properties.Schema.LIST:
-            para = nodes.emphasis(
-                '', _('List contents:'))
+            para = nodes.paragraph()
+            emph = nodes.emphasis('', _('List contents:'))
+            para.append(emph)
             definition.append(para)
             sub_schema = prop.schema
 
@@ -262,14 +289,13 @@ Resources:
             self.contribute_property(prop_list, prop_key, prop)
 
     def contribute_attributes(self, parent):
-        schema = self.resource_class.attributes_schema
-        if not schema:
+        if not self.attrs_schemata:
             return
         section = self._section(parent, _('Attributes'), '%s-attrs')
         prop_list = nodes.definition_list()
         section.append(prop_list)
-        for prop_key in sorted(schema.keys()):
-            description = schema[prop_key]
+        for prop_key, prop in sorted(self.attrs_schemata.items()):
+            description = prop.description
             prop_item = nodes.definition_list_item(
                 '', nodes.term('', prop_key))
             prop_list.append(prop_item)
@@ -277,6 +303,15 @@ Resources:
             definition = nodes.definition()
             prop_item.append(definition)
 
+            if prop.support_status.status != support.SUPPORTED:
+                sstatus = prop.support_status.to_dict()
+                msg = _('%(status)s')
+                if sstatus['message'] is not None:
+                    msg = _('%(status)s - %(message)s')
+                para = nodes.inline('', msg % sstatus)
+                warning = nodes.note('', para)
+                definition.append(warning)
+
             if description:
                 def_para = nodes.paragraph('', description)
                 definition.append(def_para)
