@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -22,17 +21,18 @@ import socket
 from heat.api.aws import exception
 from heat.api.aws import utils as api_utils
 from heat.common import exception as heat_exception
+from heat.common.i18n import _
+from heat.common.i18n import _LI
 from heat.common import identifier
 from heat.common import policy
 from heat.common import template_format
 from heat.common import urlfetch
 from heat.common import wsgi
-from heat.openstack.common.gettextutils import _
 from heat.openstack.common import log as logging
 from heat.rpc import api as engine_api
 from heat.rpc import client as rpc_client
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class StackController(object):
@@ -247,15 +247,15 @@ class StackController(object):
         Get template file contents, either from local file or URL
         """
         if 'TemplateBody' in req.params:
-            logger.debug('TemplateBody ...')
+            LOG.debug('TemplateBody ...')
             return req.params['TemplateBody']
         elif 'TemplateUrl' in req.params:
             url = req.params['TemplateUrl']
-            logger.debug('TemplateUrl %s' % url)
+            LOG.debug('TemplateUrl %s' % url)
             try:
                 return urlfetch.get(url)
             except IOError as exc:
-                msg = _('Failed to fetch template: %s') % str(exc)
+                msg = _('Failed to fetch template: %s') % exc
                 raise exception.HeatInvalidParameterValueError(detail=msg)
 
         return None
@@ -365,6 +365,20 @@ class StackController(object):
 
         return api_utils.format_response(action, response)
 
+    def cancel_update(self, req):
+        action = 'CancelUpdateStack'
+        self._enforce(req, action)
+        con = req.context
+        stack_name = req.params['StackName']
+        stack_identity = self._get_identity(con, stack_name)
+        try:
+            self.rpc_client.stack_cancel_update(
+                con, stack_identity=stack_identity)
+        except Exception as ex:
+            return exception.map_remote_error(ex)
+
+        return api_utils.format_response(action, {})
+
     def get_template(self, req):
         """
         Implements the GetTemplate API action.
@@ -422,7 +436,7 @@ class StackController(object):
             msg = _("The Template must be a JSON or YAML document.")
             return exception.HeatInvalidParameterValueError(detail=msg)
 
-        logger.info('validate_template')
+        LOG.info(_LI('validate_template'))
 
         def format_validate_parameter(key, value):
             """
