@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -22,9 +21,6 @@ from heat.api.openstack.v1 import software_configs
 from heat.api.openstack.v1 import software_deployments
 from heat.api.openstack.v1 import stacks
 from heat.common import wsgi
-from heat.openstack.common import log as logging
-
-logger = logging.getLogger(__name__)
 
 
 class API(wsgi.Router):
@@ -110,6 +106,10 @@ class API(wsgi.Router):
                                  "/stacks/{stack_name}/{stack_id}",
                                  action="update",
                                  conditions={'method': 'PUT'})
+            stack_mapper.connect("stack_update_patch",
+                                 "/stacks/{stack_name}/{stack_id}",
+                                 action="update_patch",
+                                 conditions={'method': 'PATCH'})
             stack_mapper.connect("stack_delete",
                                  "/stacks/{stack_name}/{stack_id}",
                                  action="delete",
@@ -121,6 +121,28 @@ class API(wsgi.Router):
                                  action="abandon",
                                  conditions={'method': 'DELETE'})
 
+            stack_mapper.connect("stack_snapshot",
+                                 "/stacks/{stack_name}/{stack_id}/snapshots",
+                                 action="snapshot",
+                                 conditions={'method': 'POST'})
+
+            stack_mapper.connect("stack_snapshot_show",
+                                 "/stacks/{stack_name}/{stack_id}/snapshots/"
+                                 "{snapshot_id}",
+                                 action="show_snapshot",
+                                 conditions={'method': 'GET'})
+
+            stack_mapper.connect("stack_snapshot_delete",
+                                 "/stacks/{stack_name}/{stack_id}/snapshots/"
+                                 "{snapshot_id}",
+                                 action="delete_snapshot",
+                                 conditions={'method': 'DELETE'})
+
+            stack_mapper.connect("stack_list_snapshots",
+                                 "/stacks/{stack_name}/{stack_id}/snapshots",
+                                 action="list_snapshots",
+                                 conditions={'method': 'GET'})
+
         # Resources
         resources_resource = resources.create_resource(conf)
         stack_path = "/{tenant_id}/stacks/{stack_name}/{stack_id}"
