@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -35,7 +35,7 @@ def init_logging():
     LOG.setLevel(logging.INFO)
     LOG.addHandler(logging.StreamHandler())
     fh = logging.FileHandler("/var/log/heat-provision.log")
-    os.chmod(fh.baseFilename, int("600", 8))
+    os.chmod(fh.baseFilename, 0o600)
     LOG.addHandler(fh)
 
 
@@ -46,7 +46,7 @@ def call(args):
         def write(self, data):
             LOG.info(data)
 
-    LOG.info('%s\n' % ' '.join(args))
+    LOG.info('%s\n', ' '.join(args))  # noqa
     try:
         ls = LogStream()
         p = subprocess.Popen(args, stdout=subprocess.PIPE,
@@ -55,17 +55,15 @@ def call(args):
         if data:
             for x in data:
                 ls.write(x)
-    except OSError:
-        ex_type, ex, tb = sys.exc_info()
+    except OSError as ex:
         if ex.errno == errno.ENOEXEC:
-            LOG.error('Userdata empty or not executable: %s\n' % str(ex))
+            LOG.error('Userdata empty or not executable: %s', ex)
             return os.EX_OK
         else:
-            LOG.error('OS error running userdata: %s\n' % str(ex))
+            LOG.error('OS error running userdata: %s', ex)
             return os.EX_OSERR
-    except Exception:
-        ex_type, ex, tb = sys.exc_info()
-        LOG.error('Unknown error running userdata: %s\n' % str(ex))
+    except Exception as ex:
+        LOG.error('Unknown error running userdata: %s', ex)
         return os.EX_SOFTWARE
     return p.returncode
 
@@ -75,15 +73,15 @@ def main():
     if not chk_ci_version():
         # pre 0.6.0 - user data executed via cloudinit, not this helper
         LOG.error('Unable to log provisioning, need a newer version of'
-                  ' cloud-init\n')
+                  ' cloud-init')
         return -1
 
     userdata_path = os.path.join(VAR_PATH, 'cfn-userdata')
-    os.chmod(userdata_path, int("700", 8))
+    os.chmod(userdata_path, 0o700)
 
-    LOG.info('Provision began: %s\n' % datetime.datetime.now())
+    LOG.info('Provision began: %s', datetime.datetime.now())
     returncode = call([userdata_path])
-    LOG.info('Provision done: %s\n' % datetime.datetime.now())
+    LOG.info('Provision done: %s', datetime.datetime.now())
     if returncode:
         return returncode
 
@@ -93,13 +91,10 @@ if __name__ == '__main__':
 
     code = main()
     if code:
-        LOG.error('Provision failed with exit code %s' % code)
+        LOG.error('Provision failed with exit code %s', code)
         sys.exit(code)
 
     provision_log = os.path.join(VAR_PATH, 'provision-finished')
     # touch the file so it is timestamped with when finished
-    pl = file(provision_log, 'a')
-    try:
+    with file(provision_log, 'a'):
         os.utime(provision_log, None)
-    finally:
-        pl.close()
