@@ -1,5 +1,5 @@
 #part-handler
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -15,7 +15,6 @@
 import datetime
 import errno
 import os
-import sys
 
 
 def list_types():
@@ -25,9 +24,8 @@ def list_types():
 def handle_part(data, ctype, filename, payload):
     if ctype == "__begin__":
         try:
-            os.makedirs('/var/lib/heat-cfntools', int("700", 8))
-        except OSError:
-            ex_type, e, tb = sys.exc_info()
+            os.makedirs('/var/lib/heat-cfntools', 0o700)
+        except OSError as e:
             if e.errno != errno.EEXIST:
                 raise
         return
@@ -35,23 +33,14 @@ def handle_part(data, ctype, filename, payload):
     if ctype == "__end__":
         return
 
-    log = open('/var/log/part-handler.log', 'a')
-    try:
+    with open('/var/log/part-handler.log', 'a') as log:
         timestamp = datetime.datetime.now()
         log.write('%s filename:%s, ctype:%s\n' % (timestamp, filename, ctype))
-    finally:
-        log.close()
 
     if ctype == 'text/x-cfninitdata':
-        f = open('/var/lib/heat-cfntools/%s' % filename, 'w')
-        try:
+        with open('/var/lib/heat-cfntools/%s' % filename, 'w') as f:
             f.write(payload)
-        finally:
-            f.close()
 
         # TODO(sdake) hopefully temporary until users move to heat-cfntools-1.3
-        f = open('/var/lib/cloud/data/%s' % filename, 'w')
-        try:
+        with open('/var/lib/cloud/data/%s' % filename, 'w') as f:
             f.write(payload)
-        finally:
-            f.close()
