@@ -13,7 +13,7 @@
 
 import sqlalchemy
 
-from heat.db.sqlalchemy import types
+from heat.db.sqlalchemy.types import Json
 
 
 def upgrade(migrate_engine):
@@ -34,7 +34,7 @@ def upgrade(migrate_engine):
         sqlalchemy.Column('updated_at', sqlalchemy.DateTime),
         sqlalchemy.Column('status', sqlalchemy.String(255)),
         sqlalchemy.Column('status_reason', sqlalchemy.String(255)),
-        sqlalchemy.Column('data', types.Json),
+        sqlalchemy.Column('data', Json),
         sqlalchemy.Column('tenant', sqlalchemy.String(64),
                           nullable=False,
                           index=True),
@@ -43,3 +43,11 @@ def upgrade(migrate_engine):
     )
     sqlalchemy.Table('stack', meta, autoload=True)
     snapshot.create()
+
+
+def downgrade(migrate_engine):
+    meta = sqlalchemy.MetaData()
+    meta.bind = migrate_engine
+
+    snapshot = sqlalchemy.Table('snapshot', meta, autoload=True)
+    snapshot.drop()
