@@ -13,8 +13,9 @@
 
 from json import dumps
 from json import loads
-from sqlalchemy import types
+
 from sqlalchemy.dialects import mysql
+from sqlalchemy import types
 
 
 class LongText(types.TypeDecorator):
@@ -33,6 +34,8 @@ class Json(LongText):
         return dumps(value)
 
     def process_result_value(self, value, dialect):
+        if value is None:
+            return None
         return loads(value)
 
 
