@@ -11,15 +11,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo.utils import timeutils
+
+from heat.common.i18n import _
+from heat.common import param_utils
 from heat.common import template_format
-from heat.rpc import api
-from heat.openstack.common import timeutils
 from heat.engine import constraints as constr
-
 from heat.openstack.common import log as logging
-from heat.openstack.common.gettextutils import _
+from heat.rpc import api
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 def extract_args(params):
@@ -34,7 +35,7 @@ def extract_args(params):
         try:
             timeout = int(timeout_mins)
         except (ValueError, TypeError):
-            logger.exception(_('Timeout conversion failed'))
+            LOG.exception(_('Timeout conversion failed'))
         else:
             if timeout > 0:
                 kwargs[api.PARAM_TIMEOUT] = timeout
@@ -42,16 +43,13 @@ def extract_args(params):
                 raise ValueError(_('Invalid timeout value %s') % timeout)
 
     if api.PARAM_DISABLE_ROLLBACK in params:
-        disable_rollback = params.get(api.PARAM_DISABLE_ROLLBACK)
-        if str(disable_rollback).lower() == 'true':
-            kwargs[api.PARAM_DISABLE_ROLLBACK] = True
-        elif str(disable_rollback).lower() == 'false':
-            kwargs[api.PARAM_DISABLE_ROLLBACK] = False
-        else:
-            raise ValueError(_('Unexpected value for parameter'
-                               ' %(name)s : %(value)s') %
-                             dict(name=api.PARAM_DISABLE_ROLLBACK,
-                                  value=disable_rollback))
+        disable_rollback = param_utils.extract_bool(
+            params[api.PARAM_DISABLE_ROLLBACK])
+        kwargs[api.PARAM_DISABLE_ROLLBACK] = disable_rollback
+
+    if api.PARAM_SHOW_DELETED in params:
+        params[api.PARAM_SHOW_DELETED] = param_utils.extract_bool(
+            params[api.PARAM_SHOW_DELETED])
 
     adopt_data = params.get(api.PARAM_ADOPT_STACK_DATA)
     if adopt_data:
@@ -71,15 +69,20 @@ def format_stack_outputs(stack, outputs):
     that matches the API output expectations.
     '''
     def format_stack_output(k):
-        return {api.OUTPUT_DESCRIPTION: outputs[k].get('Description',
-                                                       'No description given'),
-                api.OUTPUT_KEY: k,
-                api.OUTPUT_VALUE: stack.output(k)}
+        output = {
+            api.OUTPUT_DESCRIPTION: outputs[k].get('Description',
+                                                   'No description given'),
+            api.OUTPUT_KEY: k,
+            api.OUTPUT_VALUE: stack.output(k)
+        }
+        if outputs[k].get('error_msg'):
+            output.update({api.OUTPUT_ERROR: outputs[k].get('error_msg')})
+        return output
 
     return [format_stack_output(key) for key in outputs]
 
 
-def format_stack(stack):
+def format_stack(stack, preview=False):
     '''
     Return a representation of the given stack that matches the API output
     expectations.
@@ -94,22 +97,40 @@ def format_stack(stack):
         api.STACK_PARAMETERS: stack.parameters.map(str),
         api.STACK_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
         api.STACK_TMPL_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
-        api.STACK_ACTION: stack.action or '',
-        api.STACK_STATUS: stack.status or '',
-        api.STACK_STATUS_DATA: stack.status_reason,
         api.STACK_CAPABILITIES: [],   # TODO Not implemented yet
         api.STACK_DISABLE_ROLLBACK: stack.disable_rollback,
         api.STACK_TIMEOUT: stack.timeout_mins,
+        api.STACK_OWNER: stack.username,
+        api.STACK_PARENT: stack.owner_id,
     }
 
-    # only show the outputs on a completely created or updated stack
-    if (stack.action != stack.DELETE and stack.status == stack.COMPLETE):
+    if not preview:
+        update_info = {
+            api.STACK_ACTION: stack.action or '',
+            api.STACK_STATUS: stack.status or '',
+            api.STACK_STATUS_DATA: stack.status_reason,
+        }
+        info.update(update_info)
+
+    # allow users to view the outputs of stacks
+    if (stack.action != stack.DELETE and stack.status != stack.IN_PROGRESS):
         info[api.STACK_OUTPUTS] = format_stack_outputs(stack, stack.outputs)
 
     return info
 
 
-def format_stack_resource(resource, detail=True):
+def format_resource_properties(resource):
+    def get_property(prop):
+        try:
+            return resource.properties[prop]
+        except (KeyError, ValueError):
+            return None
+
+    return dict((prop, get_property(prop))
+                for prop in resource.properties_schema.keys())
+
+
+def format_stack_resource(resource, detail=True, with_props=False):
     '''
     Return a representation of the given resource that matches the API output
     expectations.
@@ -119,20 +140,29 @@ def format_stack_resource(resource, detail=True):
         api.RES_UPDATED_TIME: timeutils.isotime(last_updated_time),
         api.RES_NAME: resource.name,
         api.RES_PHYSICAL_ID: resource.resource_id or '',
-        api.RES_METADATA: resource.metadata,
         api.RES_ACTION: resource.action,
         api.RES_STATUS: resource.status,
         api.RES_STATUS_DATA: resource.status_reason,
-        api.RES_TYPE: resource.t['Type'],
+        api.RES_TYPE: resource.type(),
         api.RES_ID: dict(resource.identifier()),
         api.RES_STACK_ID: dict(resource.stack.identifier()),
         api.RES_STACK_NAME: resource.stack.name,
         api.RES_REQUIRED_BY: resource.required_by(),
     }
 
+    if (hasattr(resource, 'nested') and callable(resource.nested) and
+            resource.nested()):
+        res[api.RES_NESTED_STACK_ID] = dict(resource.nested().identifier())
+
+    if resource.stack.parent_resource:
+        res[api.RES_PARENT_RESOURCE] = resource.stack.parent_resource.name
+
     if detail:
-        res[api.RES_DESCRIPTION] = resource.parsed_template('Description', '')
-        res[api.RES_METADATA] = resource.metadata
+        res[api.RES_DESCRIPTION] = resource.t.description
+        res[api.RES_METADATA] = resource.metadata_get()
+
+    if with_props:
+        res[api.RES_SCHEMA_PROPERTIES] = format_resource_properties(resource)
 
     return res
 
@@ -141,9 +171,9 @@ def format_stack_preview(stack):
     def format_resource(res):
         if isinstance(res, list):
             return map(format_resource, res)
-        return format_stack_resource(res)
+        return format_stack_resource(res, with_props=True)
 
-    fmt_stack = format_stack(stack)
+    fmt_stack = format_stack(stack, preview=True)
     fmt_resources = map(format_resource, stack.preview_resources())
     fmt_stack['resources'] = fmt_resources
 
@@ -171,7 +201,7 @@ def format_event(event):
 
 
 def format_notification_body(stack):
-    # some other posibilities here are:
+    # some other possibilities here are:
     # - template name
     # - template size
     # - resource count
@@ -234,7 +264,7 @@ def format_watch_data(wd):
     if len(metric) == 1:
         metric_name, metric_data = metric[0]
     else:
-        logger.error(_("Unexpected number of keys in watch_data.data!"))
+        LOG.error(_("Unexpected number of keys in watch_data.data!"))
         return
 
     result = {
@@ -263,7 +293,8 @@ def format_validate_parameter(param):
         param.schema.STRING: api.PARAM_TYPE_STRING,
         param.schema.NUMBER: api.PARAM_TYPE_NUMBER,
         param.schema.LIST: api.PARAM_TYPE_COMMA_DELIMITED_LIST,
-        param.schema.MAP: api.PARAM_TYPE_JSON
+        param.schema.MAP: api.PARAM_TYPE_JSON,
+        param.schema.BOOLEAN: api.PARAM_TYPE_BOOLEAN
     }
 
     res = {
@@ -301,6 +332,9 @@ def format_validate_parameter(param):
         elif isinstance(c, constr.AllowedPattern):
             res[api.PARAM_ALLOWED_PATTERN] = c.pattern
 
+        elif isinstance(c, constr.CustomConstraint):
+            res[api.PARAM_CUSTOM_CONSTRAINT] = c.name
+
         if c.description:
             constraint_description.append(c.description)
 
@@ -340,3 +374,16 @@ def format_software_deployment(sd):
         api.SOFTWARE_DEPLOYMENT_CONFIG_ID: sd.config.id,
     }
     return result
+
+
+def format_snapshot(snapshot):
+    if snapshot is None:
+        return
+    result = {
+        api.SNAPSHOT_ID: snapshot.id,
+        api.SNAPSHOT_NAME: snapshot.name,
+        api.SNAPSHOT_STATUS: snapshot.status,
+        api.SNAPSHOT_STATUS_REASON: snapshot.status_reason,
+        api.SNAPSHOT_DATA: snapshot.data,
+    }
+    return result
