@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,12 +12,15 @@
 #    under the License.
 
 import collections
+import itertools
 import json
 
+import six
+
 from heat.api.aws import utils as aws_utils
 from heat.common import exception
-
 from heat.engine import function
+from heat.engine import resource
 
 
 class FindInMap(function.Function):
@@ -38,7 +40,7 @@ class FindInMap(function.Function):
         try:
             self._mapname, self._mapkey, self._mapvalue = self.args
         except ValueError as ex:
-            raise KeyError(str(ex))
+            raise KeyError(six.text_type(ex))
 
     def result(self):
         mapping = self.stack.t.maps[function.resolve(self._mapname)]
@@ -85,8 +87,9 @@ class ParamRef(function.Function):
 
         try:
             return self.parameters[param_name]
-        except (KeyError, ValueError):
-            raise exception.UserParameterMissing(key=param_name)
+        except KeyError:
+            raise exception.InvalidTemplateReference(resource=param_name,
+                                                     key='unknown')
 
 
 class ResourceRef(function.Function):
@@ -98,10 +101,18 @@ class ResourceRef(function.Function):
         { "Ref" : "<resource_name>" }
     '''
 
-    def _resource(self):
+    def _resource(self, path='unknown'):
         resource_name = function.resolve(self.args)
 
-        return self.stack[resource_name]
+        try:
+            return self.stack[resource_name]
+        except KeyError:
+            raise exception.InvalidTemplateReference(resource=resource_name,
+                                                     key=path)
+
+    def dependencies(self, path):
+        return itertools.chain(super(ResourceRef, self).dependencies(path),
+                               [self._resource(path)])
 
     def result(self):
         return self._resource().FnGetRefId()
@@ -150,23 +161,33 @@ class GetAtt(function.Function):
 
         return resource_name, attribute
 
-    def _resource(self):
+    def _resource(self, path='unknown'):
         resource_name = function.resolve(self._resource_name)
 
         try:
             return self.stack[resource_name]
         except KeyError:
+            raise exception.InvalidTemplateReference(resource=resource_name,
+                                                     key=path)
+
+    def dependencies(self, path):
+        return itertools.chain(super(GetAtt, self).dependencies(path),
+                               [self._resource(path)])
+
+    def validate(self):
+        super(GetAtt, self).validate()
+        res = self._resource()
+        attr = function.resolve(self._attribute)
+        if (type(res).FnGetAtt == resource.Resource.FnGetAtt and
+                attr not in res.attributes_schema.keys()):
             raise exception.InvalidTemplateAttribute(
-                resource=resource_name,
-                key=function.resolve(self._attribute))
+                resource=self._resource_name, key=attr)
 
     def result(self):
         attribute = function.resolve(self._attribute)
 
         r = self._resource()
-        if (r.status in (r.IN_PROGRESS, r.COMPLETE) and
-                r.action in (r.CREATE, r.ADOPT, r.SUSPEND, r.RESUME,
-                             r.UPDATE)):
+        if (r.action in (r.CREATE, r.ADOPT, r.SUSPEND, r.RESUME, r.UPDATE)):
             return r.FnGetAtt(attribute)
         else:
             return None
@@ -293,7 +314,8 @@ class Join(function.Function):
             if s is None:
                 return ''
             if not isinstance(s, basestring):
-                raise TypeError(_('Items to join must be strings'))
+                raise TypeError(
+                    _('Items to join must be strings %s') % (repr(s)[:200]))
             return s
 
         return delim.join(ensure_string(s) for s in strings)
@@ -344,7 +366,7 @@ class Split(function.Function):
 
 class Replace(function.Function):
     '''
-    A function for performing string subsitutions.
+    A function for performing string substitutions.
 
     Takes the form::
 
@@ -416,7 +438,7 @@ class Replace(function.Function):
 
             return string.replace(placeholder, unicode(value))
 
-        return reduce(replace, mapping.iteritems(), template)
+        return reduce(replace, six.iteritems(mapping), template)
 
 
 class Base64(function.Function):
@@ -528,42 +550,10 @@ class ResourceFacade(function.Function):
         attr = function.resolve(self.args)
 
         if attr == self.METADATA:
-            return self.stack.parent_resource.metadata
+            return self.stack.parent_resource.metadata_get()
         elif attr == self.UPDATE_POLICY:
             up = self.stack.parent_resource.t.get('UpdatePolicy', {})
             return function.resolve(up)
         elif attr == self.DELETION_POLICY:
-            dp = self.stack.parent_resource.t.get('DeletionPolicy', 'Delete')
+            dp = self.stack.parent_resource.t.deletion_policy()
             return function.resolve(dp)
-
-
-def function_mapping(version_key, version):
-    if version_key == 'AWSTemplateFormatVersion':
-        return {
-            'Fn::FindInMap': FindInMap,
-            'Fn::GetAZs': GetAZs,
-            'Ref': Ref,
-            'Fn::GetAtt': GetAtt,
-            'Fn::Select': Select,
-            'Fn::Join': Join,
-            'Fn::Base64': Base64,
-        }
-    elif version_key != 'HeatTemplateFormatVersion':
-        return {}
-
-    if version == '2012-12-12':
-        return {
-            'Fn::FindInMap': FindInMap,
-            'Fn::GetAZs': GetAZs,
-            'Ref': Ref,
-            'Fn::GetAtt': GetAtt,
-            'Fn::Select': Select,
-            'Fn::Join': Join,
-            'Fn::Split': Split,
-            'Fn::Replace': Replace,
-            'Fn::Base64': Base64,
-            'Fn::MemberListToMap': MemberListToMap,
-            'Fn::ResourceFacade': ResourceFacade,
-        }
-
-    return {}
