@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,13 +11,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from heat.db import api as db_api
-from heat.common import exception
-from heat.common import identifier
-from heat.openstack.common import log as logging
-from heat.openstack.common.gettextutils import _
+import six
 
-logger = logging.getLogger(__name__)
+from heat.common import exception
+from heat.common.i18n import _
+from heat.common import identifier
+from heat.db import api as db_api
+from heat.openstack.common import log as logging
+
+LOG = logging.getLogger(__name__)
 
 
 class Event(object):
@@ -43,7 +44,7 @@ class Event(object):
         try:
             self.resource_properties = dict(resource_properties)
         except ValueError as ex:
-            self.resource_properties = {'Error': str(ex)}
+            self.resource_properties = {'Error': six.text_type(ex)}
         self.uuid = uuid
         self.timestamp = timestamp
         self.id = id
@@ -86,9 +87,6 @@ class Event(object):
         if self.timestamp is not None:
             ev['created_at'] = self.timestamp
 
-        if self.id is not None:
-            logger.warning(_('Duplicating event'))
-
         new_ev = db_api.event_create(self.context, ev)
         self.id = new_ev.id
         return self.id
