@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,6 +11,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import six
+
 from heat.common import exception
 from heat.engine import constraints
 from heat.engine import properties
@@ -69,6 +70,23 @@ common_properties_schema = {
 }
 
 
+NOVA_METERS = ['instance', 'memory', 'memory.usage',
+               'cpu', 'cpu_util', 'vcpus',
+               'disk.read.requests', 'disk.read.requests.rate',
+               'disk.write.requests', 'disk.write.requests.rate',
+               'disk.read.bytes', 'disk.read.bytes.rate',
+               'disk.write.bytes', 'disk.write.bytes.rate',
+               'disk.device.read.requests', 'disk.device.read.requests.rate',
+               'disk.device.write.requests', 'disk.device.write.requests.rate',
+               'disk.device.read.bytes', 'disk.device.read.bytes.rate',
+               'disk.device.write.bytes', 'disk.device.write.bytes.rate',
+               'disk.root.size', 'disk.ephemeral.size',
+               'network.incoming.bytes', 'network.incoming.bytes.rate',
+               'network.outgoing.bytes', 'network.outgoing.bytes.rate',
+               'network.incoming.packets', 'network.incoming.packets.rate',
+               'network.outgoing.packets', 'network.outgoing.packets.rate']
+
+
 def actions_to_urls(stack, properties):
     kwargs = {}
     for k, v in iter(properties.items()):
@@ -85,7 +103,8 @@ def actions_to_urls(stack, properties):
                     url = stack[act].FnGetAtt('AlarmUrl')
                     kwargs[k].append(url)
                 else:
-                    kwargs[k].append(act)
+                    if act:
+                        kwargs[k].append(act)
         else:
             kwargs[k] = v
     return kwargs
@@ -144,15 +163,37 @@ class CeilometerAlarm(resource.Resource):
         MATCHING_METADATA: properties.Schema(
             properties.Schema.MAP,
             _('Meter should match this resource metadata (key=value) '
-              'additionally to the meter_name.')
+              'additionally to the meter_name.'),
+            default={}
         ),
     }
     properties_schema.update(common_properties_schema)
 
-    update_allowed_keys = ('Properties',)
+    default_client_name = 'ceilometer'
+
+    def cfn_to_ceilometer(self, stack, properties):
+        kwargs = actions_to_urls(stack, properties)
+        if kwargs.get(self.METER_NAME) in NOVA_METERS:
+            prefix = 'user_metadata.'
+        else:
+            prefix = 'metering.'
+        for k, v in iter(properties.items()):
+            if k == self.MATCHING_METADATA:
+                # make sure we have matching_metadata that looks like this:
+                # matching_metadata: {metadata.$prefix.x}
+                kwargs[k] = {}
+                for m_k, m_v in six.iteritems(v):
+                    if m_k.startswith('metadata.%s' % prefix):
+                        kwargs[k][m_k] = m_v
+                    elif m_k.startswith(prefix):
+                        kwargs[k]['metadata.%s' % m_k] = m_v
+                    else:
+                        kwargs[k]['metadata.%s%s' % (prefix, m_k)] = m_v
+        return kwargs
 
     def handle_create(self):
-        props = actions_to_urls(self.stack, self.parsed_template('Properties'))
+        props = self.cfn_to_ceilometer(self.stack,
+                                       self.properties)
         props['name'] = self.physical_resource_name()
 
         alarm = self.ceilometer().alarms.create(**props)
@@ -174,7 +215,7 @@ class CeilometerAlarm(resource.Resource):
             kwargs = {'alarm_id': self.resource_id}
             kwargs.update(prop_diff)
             alarms_client = self.ceilometer().alarms
-            alarms_client.update(**actions_to_urls(self.stack, kwargs))
+            alarms_client.update(**self.cfn_to_ceilometer(self.stack, kwargs))
 
     def handle_suspend(self):
         if self.resource_id is not None:
@@ -195,7 +236,10 @@ class CeilometerAlarm(resource.Resource):
             pass
 
         if self.resource_id is not None:
-            self.ceilometer().alarms.delete(self.resource_id)
+            try:
+                self.ceilometer().alarms.delete(self.resource_id)
+            except Exception as ex:
+                self.client_plugin().ignore_not_found(ex)
 
 
 class CombinationAlarm(resource.Resource):
@@ -221,11 +265,11 @@ class CombinationAlarm(resource.Resource):
     }
     properties_schema.update(common_properties_schema)
 
-    update_allowed_keys = ('Properties',)
+    default_client_name = 'ceilometer'
 
     def handle_create(self):
         properties = actions_to_urls(self.stack,
-                                     self.parsed_template('Properties'))
+                                     self.properties)
         properties['name'] = self.physical_resource_name()
         properties['type'] = 'combination'
 
@@ -235,7 +279,7 @@ class CombinationAlarm(resource.Resource):
 
     def _reformat_properties(self, properties):
         combination_rule = {}
-        for name in [self.ALARM_IDS, self.OPERATOR, REPEAT_ACTIONS]:
+        for name in [self.ALARM_IDS, self.OPERATOR]:
             value = properties.pop(name, None)
             if value:
                 combination_rule[name] = value
@@ -260,7 +304,10 @@ class CombinationAlarm(resource.Resource):
             alarm_id=self.resource_id, enabled=True)
 
     def handle_delete(self):
-        self.ceilometer().alarms.delete(self.resource_id)
+        try:
+            self.ceilometer().alarms.delete(self.resource_id)
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
 
 def resource_mapping():
