@@ -224,7 +224,7 @@ class RouterInterface(neutron.NeutronResource):
         subnet_id_value = properties.get(depr_subnet_key)
         if subnet_value and subnet_id_value:
             raise exception.ResourcePropertyConflict(subnet_key,
-                                                     subnet_key)
+                                                     depr_subnet_key)
         if not subnet_value and not subnet_id_value:
             return False
         return True
