@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,95 +13,95 @@
 
 import abc
 import collections
+import copy
 import functools
+import six
+from stevedore import extension
 
-from heat.db import api as db_api
 from heat.common import exception
-from heat.engine import plugin_manager
+from heat.db import api as db_api
+from heat.openstack.common import log as logging
 
+LOG = logging.getLogger(__name__)
 
 __all__ = ['Template']
 
 
-DEFAULT_VERSION = ('HeatTemplateFormatVersion', '2012-12-12')
-
 _template_classes = None
 
 
-class TemplatePluginManager(object):
-    '''A Descriptor class for caching PluginManagers.
-
-    Keeps a cache of PluginManagers with the search directories corresponding
-    to the package containing the owner class.
-    '''
-
-    def __init__(self):
-        self.plugin_managers = {}
-
-    @staticmethod
-    def package_name(obj_class):
-        '''Return the package containing the given class.'''
-        module_name = obj_class.__module__
-        return module_name.rsplit('.', 1)[0]
-
-    def __get__(self, obj, obj_class):
-        '''Get a PluginManager for a class.'''
-        pkg = self.package_name(obj_class)
-        if pkg not in self.plugin_managers:
-            self.plugin_managers[pkg] = plugin_manager.PluginManager(pkg)
-
-        return self.plugin_managers[pkg]
-
-
 def get_version(template_data, available_versions):
     version_keys = set(key for key, version in available_versions)
-    candidate_keys = set(k for k, v in template_data.iteritems() if
+    candidate_keys = set(k for k, v in six.iteritems(template_data) if
                          isinstance(v, basestring))
 
     keys_present = version_keys & candidate_keys
-    if not keys_present:
-        return DEFAULT_VERSION
 
     if len(keys_present) > 1:
         explanation = _('Ambiguous versions (%s)') % ', '.join(keys_present)
         raise exception.InvalidTemplateVersion(explanation=explanation)
-
-    version_key = keys_present.pop()
+    try:
+        version_key = keys_present.pop()
+    except KeyError:
+        explanation = _('Template version was not provided')
+        raise exception.InvalidTemplateVersion(explanation=explanation)
     return version_key, template_data[version_key]
 
 
-def get_template_class(plugin_mgr, template_data):
+def _get_template_extension_manager():
+    return extension.ExtensionManager(
+        namespace='heat.templates',
+        invoke_on_load=False,
+        verify_requirements=True,
+        on_load_failure_callback=raise_extension_exception)
+
+
+def raise_extension_exception(extmanager, ep, err):
+    raise TemplatePluginNotRegistered(name=ep.name, error=six.text_type(err))
+
+
+class TemplatePluginNotRegistered(exception.HeatException):
+    msg_fmt = _("Could not load %(name)s: %(error)s")
+
+
+def get_template_class(template_data):
     global _template_classes
 
     if _template_classes is None:
-        tmpl_mapping = plugin_manager.PluginMapping('template')
-        _template_classes = dict(tmpl_mapping.load_all(plugin_mgr))
+        mgr = _get_template_extension_manager()
+        _template_classes = dict((tuple(name.split('.')), mgr[name].plugin)
+                                 for name in mgr.names())
 
     available_versions = _template_classes.keys()
     version = get_version(template_data, available_versions)
+    version_type = version[0]
     try:
         return _template_classes[version]
     except KeyError:
+        av_list = [v for k, v in available_versions if k == version_type]
         msg_data = {'version': ': '.join(version),
-                    'available': ', '.join(v for vk, v in available_versions)}
-        explanation = _('Unknown version (%(version)s). '
-                        'Should be one of: %(available)s') % msg_data
+                    'version_type': version_type,
+                    'available': ', '.join(v for v in av_list)}
+
+        if len(av_list) > 1:
+            explanation = _('"%(version)s". "%(version_type)s" '
+                            'should be one of: %(available)s') % msg_data
+        else:
+            explanation = _('"%(version)s". "%(version_type)s" '
+                            'should be: %(available)s') % msg_data
         raise exception.InvalidTemplateVersion(explanation=explanation)
 
 
 class Template(collections.Mapping):
     '''A stack template.'''
 
-    _plugins = TemplatePluginManager()
-    _functionmaps = {}
-
     def __new__(cls, template, *args, **kwargs):
         '''Create a new Template of the appropriate class.'''
 
         if cls != Template:
             TemplateClass = cls
         else:
-            TemplateClass = get_template_class(cls._plugins, template)
+            TemplateClass = get_template_class(template)
 
         return super(Template, cls).__new__(TemplateClass)
 
@@ -116,21 +115,27 @@ class Template(collections.Mapping):
         self.maps = self[self.MAPPINGS]
         self.version = get_version(self.t, _template_classes.keys())
 
+    def __deepcopy__(self, memo):
+        return Template(copy.deepcopy(self.t, memo), files=self.files)
+
     @classmethod
-    def load(cls, context, template_id):
+    def load(cls, context, template_id, t=None):
         '''Retrieve a Template with the given ID from the database.'''
-        t = db_api.raw_template_get(context, template_id)
+        if t is None:
+            t = db_api.raw_template_get(context, template_id)
         return cls(t.template, template_id=template_id, files=t.files)
 
     def store(self, context=None):
         '''Store the Template in the database and return its ID.'''
+        rt = {
+            'template': self.t,
+            'files': self.files
+        }
         if self.id is None:
-            rt = {
-                'template': self.t,
-                'files': self.files
-            }
             new_rt = db_api.raw_template_create(context, rt)
             self.id = new_rt.id
+        else:
+            db_api.raw_template_update(context, self.id, rt)
         return self.id
 
     def __iter__(self):
@@ -152,41 +157,68 @@ class Template(collections.Mapping):
         '''Return a parameters.Parameters object for the stack.'''
         pass
 
-    def functions(self):
-        '''Return a dict of template functions keyed by name.'''
-        if self.version not in self._functionmaps:
-            mappings = plugin_manager.PluginMapping('function', *self.version)
-            funcs = dict(mappings.load_all(self._plugins))
-            self._functionmaps[self.version] = funcs
+    @abc.abstractmethod
+    def resource_definitions(self, stack):
+        '''Return a dictionary of ResourceDefinition objects.'''
+        pass
 
-        return self._functionmaps[self.version]
+    @abc.abstractmethod
+    def add_resource(self, definition, name=None):
+        '''Add a resource to the template.
+
+        The resource is passed as a ResourceDefinition object. If no name is
+        specified, the name from the ResourceDefinition should be used.
+        '''
+        pass
+
+    def remove_resource(self, name):
+        '''Remove a resource from the template.'''
+        self.t.get(self.RESOURCES, {}).pop(name)
 
     def parse(self, stack, snippet):
-        return parse(self.functions(), stack, snippet)
+        return parse(self.functions, stack, snippet)
 
     def validate(self):
         '''Validate the template.
 
-        Only validates the top-level sections of the template. Syntax inside
-        sections is not checked here but in code parts that are responsible
-        for working with the respective sections.
+        Validates the top-level sections of the template as well as syntax
+        inside select sections. Some sections are not checked here but in
+        code parts that are responsible for working with the respective
+        sections (e.g. parameters are check by parameters schema class).
+
         '''
 
+        # check top-level sections
         for k in self.t.keys():
             if k not in self.SECTIONS:
                 raise exception.InvalidTemplateSection(section=k)
 
+        # check resources
+        for res in self[self.RESOURCES].values():
+            try:
+                if not res.get('Type'):
+                    message = _('Every Resource object must '
+                                'contain a Type member.')
+                    raise exception.StackValidationFailed(message=message)
+            except AttributeError:
+                type_res = type(res)
+                if isinstance(res, unicode):
+                    type_res = "string"
+                message = _('Resources must contain Resource. '
+                            'Found a [%s] instead') % type_res
+                raise exception.StackValidationFailed(message=message)
+
 
 def parse(functions, stack, snippet):
     recurse = functools.partial(parse, functions, stack)
 
     if isinstance(snippet, collections.Mapping):
         if len(snippet) == 1:
-            fn_name, args = next(snippet.iteritems())
+            fn_name, args = next(six.iteritems(snippet))
             Func = functions.get(fn_name)
             if Func is not None:
                 return Func(stack, fn_name, recurse(args))
-        return dict((k, recurse(v)) for k, v in snippet.iteritems())
+        return dict((k, recurse(v)) for k, v in six.iteritems(snippet))
     elif (not isinstance(snippet, basestring) and
           isinstance(snippet, collections.Iterable)):
         return [recurse(v) for v in snippet]
