@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,14 +13,15 @@
 
 
 import json
+import six
 
 from oslo.config import cfg
+from oslo.utils import importutils
 import requests
 
 from heat.api.aws import ec2token
 from heat.api.aws import exception
 from heat.common.wsgi import Request
-from heat.openstack.common import importutils
 from heat.tests.common import HeatTestCase
 
 
@@ -33,8 +34,10 @@ class Ec2TokenTest(HeatTestCase):
         super(Ec2TokenTest, self).setUp()
         self.m.StubOutWithMock(requests, 'post')
 
-    def _dummy_GET_request(self, params={}, environ={}):
+    def _dummy_GET_request(self, params=None, environ=None):
         # Mangle the params dict into a query string
+        params = params or {}
+        environ = environ or {}
         qs = "&".join(["=".join([k, str(params[k])]) for k in params])
         environ.update({'REQUEST_METHOD': 'GET', 'QUERY_STRING': qs})
         req = Request(environ)
@@ -179,9 +182,12 @@ class Ec2TokenTest(HeatTestCase):
         ec2 = ec2token.EC2Token(app='xyz', conf={})
         self.assertEqual('xyz', ec2.__call__(dummy_req))
 
-    def _stub_http_connection(self, headers={}, params={}, response=None,
+    def _stub_http_connection(self, headers=None, params=None, response=None,
                               req_url='http://123:5000/v2.0/ec2tokens'):
 
+        headers = headers or {}
+        params = params or {}
+
         class DummyHTTPResponse(object):
             text = response
 
@@ -444,7 +450,7 @@ class Ec2TokenTest(HeatTestCase):
         self.m.ReplayAll()
         ex = self.assertRaises(exception.HeatInternalFailureError,
                                ec2.__call__, dummy_req)
-        self.assertEqual('Service misconfigured', str(ex))
+        self.assertEqual('Service misconfigured', six.text_type(ex))
 
         self.m.VerifyAll()
 
@@ -469,9 +475,31 @@ class Ec2TokenTest(HeatTestCase):
 
         self.m.VerifyAll()
 
+    def test_call_ok_auth_uri_ec2authtoken_long(self):
+        # Prove we tolerate a url which already includes the /ec2tokens path
+        dummy_url = 'http://123:5000/v2.0/ec2tokens'
+        cfg.CONF.set_default('auth_uri', dummy_url, group='ec2authtoken')
+
+        ec2 = ec2token.EC2Token(app='woot', conf={})
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
+            'id': 123,
+            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        self._stub_http_connection(response=ok_resp,
+                                   params={'AWSAccessKeyId': 'foo'})
+        self.m.ReplayAll()
+        self.assertEqual('woot', ec2.__call__(dummy_req))
+
+        self.m.VerifyAll()
+
     def test_call_ok_auth_uri_ks_authtoken(self):
         # Import auth_token to have keystone_authtoken settings setup.
-        importutils.import_module('keystoneclient.middleware.auth_token')
+        importutils.import_module('keystonemiddleware.auth_token')
         dummy_url = 'http://123:5000/v2.0'
         cfg.CONF.set_override('auth_uri', dummy_url,
                               group='keystone_authtoken')
