@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -12,10 +12,11 @@
 #    under the License.
 
 from datetime import datetime
-import json
 import uuid
 
+import json
 import mock
+import six
 
 from heat.common.identifier import EventIdentifier
 from heat.common import template_format
@@ -30,89 +31,12 @@ from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
 
-class EngineApiTest(HeatTestCase):
-    def test_timeout_extract(self):
-        p = {'timeout_mins': '5'}
-        args = api.extract_args(p)
-        self.assertEqual(5, args['timeout_mins'])
-
-    def test_timeout_extract_zero(self):
-        p = {'timeout_mins': '0'}
-        args = api.extract_args(p)
-        self.assertNotIn('timeout_mins', args)
-
-    def test_timeout_extract_garbage(self):
-        p = {'timeout_mins': 'wibble'}
-        args = api.extract_args(p)
-        self.assertNotIn('timeout_mins', args)
-
-    def test_timeout_extract_none(self):
-        p = {'timeout_mins': None}
-        args = api.extract_args(p)
-        self.assertNotIn('timeout_mins', args)
-
-    def test_timeout_extract_negative(self):
-        p = {'timeout_mins': '-100'}
-        error = self.assertRaises(ValueError, api.extract_args, p)
-        self.assertIn('Invalid timeout value', str(error))
-
-    def test_timeout_extract_not_present(self):
-        args = api.extract_args({})
-        self.assertNotIn('timeout_mins', args)
-
-    def test_adopt_stack_data_extract_present(self):
-        p = {'adopt_stack_data': json.dumps({'Resources': {}})}
-        args = api.extract_args(p)
-        self.assertTrue(args.get('adopt_stack_data'))
-
-    def test_invalid_adopt_stack_data(self):
-        p = {'adopt_stack_data': json.dumps("foo")}
-        error = self.assertRaises(ValueError, api.extract_args, p)
-        self.assertEqual(
-            'Unexpected adopt data "foo". Adopt data must be a dict.',
-            str(error))
-
-    def test_adopt_stack_data_extract_not_present(self):
-        args = api.extract_args({})
-        self.assertNotIn('adopt_stack_data', args)
-
-    def test_disable_rollback_extract_true(self):
-        args = api.extract_args({'disable_rollback': True})
-        self.assertIn('disable_rollback', args)
-        self.assertTrue(args.get('disable_rollback'))
-
-        args = api.extract_args({'disable_rollback': 'True'})
-        self.assertIn('disable_rollback', args)
-        self.assertTrue(args.get('disable_rollback'))
-
-        args = api.extract_args({'disable_rollback': 'true'})
-        self.assertIn('disable_rollback', args)
-        self.assertTrue(args.get('disable_rollback'))
-
-    def test_disable_rollback_extract_false(self):
-        args = api.extract_args({'disable_rollback': False})
-        self.assertIn('disable_rollback', args)
-        self.assertFalse(args.get('disable_rollback'))
-
-        args = api.extract_args({'disable_rollback': 'False'})
-        self.assertIn('disable_rollback', args)
-        self.assertFalse(args.get('disable_rollback'))
-
-        args = api.extract_args({'disable_rollback': 'false'})
-        self.assertIn('disable_rollback', args)
-        self.assertFalse(args.get('disable_rollback'))
-
-    def test_disable_rollback_extract_bad(self):
-        self.assertRaises(ValueError, api.extract_args,
-                          {'disable_rollback': 'bad'})
-
-
 class FormatTest(HeatTestCase):
     def setUp(self):
         super(FormatTest, self).setUp()
-        utils.setup_dummy_db()
 
         template = parser.Template({
+            'HeatTemplateFormatVersion': '2012-12-12',
             'Resources': {
                 'generic1': {'Type': 'GenericResourceType'},
                 'generic2': {
@@ -122,6 +46,8 @@ class FormatTest(HeatTestCase):
         })
         resource._register_class('GenericResourceType',
                                  generic_rsrc.GenericResource)
+        resource._register_class('ResWithComplexPropsAndAttrs',
+                                 generic_rsrc.ResWithComplexPropsAndAttrs)
         self.stack = parser.Stack(utils.dummy_context(), 'test_stack',
                                   template, stack_id=str(uuid.uuid4()))
 
@@ -140,7 +66,6 @@ class FormatTest(HeatTestCase):
             rpc_api.RES_UPDATED_TIME,
             rpc_api.RES_NAME,
             rpc_api.RES_PHYSICAL_ID,
-            rpc_api.RES_METADATA,
             rpc_api.RES_ACTION,
             rpc_api.RES_STATUS,
             rpc_api.RES_STATUS_DATA,
@@ -159,12 +84,93 @@ class FormatTest(HeatTestCase):
         formatted = api.format_stack_resource(res, False)
         self.assertEqual(resource_keys, set(formatted.keys()))
 
+    @mock.patch.object(api, 'format_resource_properties')
+    def test_format_stack_resource_with_props(self, mock_format_props):
+        mock_format_props.return_value = 'formatted_res_props'
+        res = self.stack['generic1']
+
+        formatted = api.format_stack_resource(res, True, with_props=True)
+        formatted_props = formatted[rpc_api.RES_SCHEMA_PROPERTIES]
+        self.assertEqual('formatted_res_props', formatted_props)
+
+    def _get_formatted_resource_properties(self, res_name):
+        tmpl = parser.Template(template_format.parse('''
+            heat_template_version: 2013-05-23
+            resources:
+              resource1:
+                type: ResWithComplexPropsAndAttrs
+              resource2:
+                type: ResWithComplexPropsAndAttrs
+                properties:
+                  a_string: foobar
+              resource3:
+                type: ResWithComplexPropsAndAttrs
+                properties:
+                  a_string: { get_attr: [ resource2, string] }
+        '''))
+        stack = parser.Stack(utils.dummy_context(), 'test_stack_for_preview',
+                             tmpl, stack_id=str(uuid.uuid4()))
+        res = stack[res_name]
+        return api.format_resource_properties(res)
+
+    def test_format_resource_properties_empty(self):
+        props = self._get_formatted_resource_properties('resource1')
+        self.assertIsNone(props['a_string'])
+        self.assertIsNone(props['a_list'])
+        self.assertIsNone(props['a_map'])
+
+    def test_format_resource_properties_direct_props(self):
+        props = self._get_formatted_resource_properties('resource2')
+        self.assertEqual('foobar', props['a_string'])
+
+    def test_format_resource_properties_get_attr(self):
+        props = self._get_formatted_resource_properties('resource3')
+        self.assertEqual('', props['a_string'])
+
+    def test_format_stack_resource_with_nested_stack(self):
+        res = self.stack['generic1']
+        nested_id = {'foo': 'bar'}
+        res.nested = mock.Mock()
+        res.nested.return_value.identifier.return_value = nested_id
+
+        formatted = api.format_stack_resource(res, False)
+        self.assertEqual(nested_id, formatted[rpc_api.RES_NESTED_STACK_ID])
+
+    def test_format_stack_resource_with_nested_stack_none(self):
+        res = self.stack['generic1']
+        res.nested = mock.Mock()
+        res.nested.return_value = None
+
+        resource_keys = set((
+            rpc_api.RES_UPDATED_TIME,
+            rpc_api.RES_NAME,
+            rpc_api.RES_PHYSICAL_ID,
+            rpc_api.RES_ACTION,
+            rpc_api.RES_STATUS,
+            rpc_api.RES_STATUS_DATA,
+            rpc_api.RES_TYPE,
+            rpc_api.RES_ID,
+            rpc_api.RES_STACK_ID,
+            rpc_api.RES_STACK_NAME,
+            rpc_api.RES_REQUIRED_BY))
+
+        formatted = api.format_stack_resource(res, False)
+        self.assertEqual(resource_keys, set(formatted.keys()))
+
     def test_format_stack_resource_required_by(self):
         res1 = api.format_stack_resource(self.stack['generic1'])
         res2 = api.format_stack_resource(self.stack['generic2'])
         self.assertEqual(['generic2'], res1['required_by'])
         self.assertEqual([], res2['required_by'])
 
+    def test_format_stack_resource_with_parent_stack(self):
+        res = self.stack['generic1']
+        res.stack.parent_resource = mock.Mock()
+        res.stack.parent_resource.name = 'foobar'
+
+        formatted = api.format_stack_resource(res, False)
+        self.assertEqual('foobar', formatted[rpc_api.RES_PARENT_RESOURCE])
+
     def test_format_event_identifier_uuid(self):
         self._test_format_event('abc123yc-9f88-404d-a85b-531529456xyz')
 
@@ -196,7 +202,7 @@ class FormatTest(HeatTestCase):
 
     @mock.patch.object(api, 'format_stack_resource')
     def test_format_stack_preview(self, mock_fmt_resource):
-        def mock_format_resources(res):
+        def mock_format_resources(res, **kwargs):
             return 'fmt%s' % res
 
         mock_fmt_resource.side_effect = mock_format_resources
@@ -205,10 +211,16 @@ class FormatTest(HeatTestCase):
 
         stack = api.format_stack_preview(self.stack)
         self.assertIsInstance(stack, dict)
+        self.assertIsNone(stack.get('status'))
+        self.assertIsNone(stack.get('action'))
+        self.assertIsNone(stack.get('status_reason'))
         self.assertEqual('test_stack', stack['stack_name'])
         self.assertIn('resources', stack)
         self.assertEqual(['fmt1', ['fmt2', ['fmt3']]], stack['resources'])
 
+        kwargs = mock_fmt_resource.call_args[1]
+        self.assertTrue(kwargs['with_props'])
+
     def test_format_stack(self):
         self.stack.created_time = datetime(1970, 1, 1)
         info = api.format_stack(self.stack)
@@ -221,9 +233,10 @@ class FormatTest(HeatTestCase):
             'description': 'No description',
             'disable_rollback': True,
             'notification_topics': [],
-            'stack_action': '',
+            'stack_action': 'CREATE',
             'stack_name': 'test_stack',
-            'stack_status': '',
+            'stack_owner': 'test_username',
+            'stack_status': 'IN_PROGRESS',
             'stack_status_reason': '',
             'template_description': 'No description',
             'timeout_mins': None,
@@ -236,7 +249,8 @@ class FormatTest(HeatTestCase):
                 'stack_id': self.stack.id,
                 'stack_name': 'test_stack',
                 'tenant': 'test_tenant_id'},
-            'updated_time': None}
+            'updated_time': None,
+            'parent': None}
         self.assertEqual(expected_stack_info, info)
 
     def test_format_stack_created_time(self):
@@ -261,6 +275,40 @@ class FormatTest(HeatTestCase):
         info = api.format_stack(self.stack)
         self.assertEqual('foobar', info[rpc_api.STACK_OUTPUTS])
 
+    def test_format_stack_outputs(self):
+        template = parser.Template({
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Resources': {
+                'generic': {'Type': 'GenericResourceType'}
+            },
+            'Outputs': {
+                'correct_output': {
+                    'Description': 'Good output',
+                    'Value': {'Fn::GetAtt': ['generic', 'Foo']}
+                },
+                'incorrect_output': {
+                    'Value': {'Fn::GetAtt': ['generic', 'Bar']}
+                }
+            }
+        })
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             template, stack_id=str(uuid.uuid4()))
+        stack.action = 'CREATE'
+        stack.status = 'COMPLETE'
+        stack['generic'].action = 'CREATE'
+        stack['generic'].status = 'COMPLETE'
+        info = api.format_stack_outputs(stack, stack.outputs)
+        expected = [{'description': 'No description given',
+                     'output_error': 'The Referenced Attribute (generic Bar) '
+                                     'is incorrect.',
+                     'output_key': 'incorrect_output',
+                     'output_value': None},
+                    {'description': 'Good output',
+                     'output_key': 'correct_output',
+                     'output_value': 'generic'}]
+
+        self.assertEqual(expected, info)
+
 
 class FormatValidateParameterTest(HeatTestCase):
 
@@ -767,6 +815,26 @@ class FormatValidateParameterTest(HeatTestCase):
                   'Label': 'KeyName'
               })
          ),
+        ('constraint_custom_hot',
+         dict(template=base_template_hot,
+              param_name='KeyName',
+              param='''
+                    "KeyName": {
+                        "type": "string",
+                        "description": "Public Network",
+                        "constraints": [
+                            { "custom_constraint": "neutron.network" }
+                        ]
+                    }
+                    ''',
+              expected={
+                  'Type': 'String',
+                  'Description': 'Public Network',
+                  'NoEcho': 'false',
+                  'Label': 'KeyName',
+                  'CustomConstraint': 'neutron.network'
+              })
+         )
     ]
 
     def test_format_validate_parameter(self):
@@ -838,3 +906,80 @@ class FormatSoftwareConfigDeploymentTest(HeatTestCase):
 
     def test_format_software_deployment_none(self):
         self.assertIsNone(api.format_software_deployment(None))
+
+
+class TestExtractArgs(HeatTestCase):
+    def test_timeout_extract(self):
+        p = {'timeout_mins': '5'}
+        args = api.extract_args(p)
+        self.assertEqual(5, args['timeout_mins'])
+
+    def test_timeout_extract_zero(self):
+        p = {'timeout_mins': '0'}
+        args = api.extract_args(p)
+        self.assertNotIn('timeout_mins', args)
+
+    def test_timeout_extract_garbage(self):
+        p = {'timeout_mins': 'wibble'}
+        args = api.extract_args(p)
+        self.assertNotIn('timeout_mins', args)
+
+    def test_timeout_extract_none(self):
+        p = {'timeout_mins': None}
+        args = api.extract_args(p)
+        self.assertNotIn('timeout_mins', args)
+
+    def test_timeout_extract_negative(self):
+        p = {'timeout_mins': '-100'}
+        error = self.assertRaises(ValueError, api.extract_args, p)
+        self.assertIn('Invalid timeout value', six.text_type(error))
+
+    def test_timeout_extract_not_present(self):
+        args = api.extract_args({})
+        self.assertNotIn('timeout_mins', args)
+
+    def test_adopt_stack_data_extract_present(self):
+        p = {'adopt_stack_data': json.dumps({'Resources': {}})}
+        args = api.extract_args(p)
+        self.assertTrue(args.get('adopt_stack_data'))
+
+    def test_invalid_adopt_stack_data(self):
+        p = {'adopt_stack_data': json.dumps("foo")}
+        error = self.assertRaises(ValueError, api.extract_args, p)
+        self.assertEqual(
+            'Unexpected adopt data "foo". Adopt data must be a dict.',
+            six.text_type(error))
+
+    def test_adopt_stack_data_extract_not_present(self):
+        args = api.extract_args({})
+        self.assertNotIn('adopt_stack_data', args)
+
+    def test_disable_rollback_extract_true(self):
+        args = api.extract_args({'disable_rollback': True})
+        self.assertIn('disable_rollback', args)
+        self.assertTrue(args.get('disable_rollback'))
+
+        args = api.extract_args({'disable_rollback': 'True'})
+        self.assertIn('disable_rollback', args)
+        self.assertTrue(args.get('disable_rollback'))
+
+        args = api.extract_args({'disable_rollback': 'true'})
+        self.assertIn('disable_rollback', args)
+        self.assertTrue(args.get('disable_rollback'))
+
+    def test_disable_rollback_extract_false(self):
+        args = api.extract_args({'disable_rollback': False})
+        self.assertIn('disable_rollback', args)
+        self.assertFalse(args.get('disable_rollback'))
+
+        args = api.extract_args({'disable_rollback': 'False'})
+        self.assertIn('disable_rollback', args)
+        self.assertFalse(args.get('disable_rollback'))
+
+        args = api.extract_args({'disable_rollback': 'false'})
+        self.assertIn('disable_rollback', args)
+        self.assertFalse(args.get('disable_rollback'))
+
+    def test_disable_rollback_extract_bad(self):
+        self.assertRaises(ValueError, api.extract_args,
+                          {'disable_rollback': 'bad'})
