@@ -12,17 +12,19 @@
 
 import copy
 import datetime
+import itertools
 
 from oslo.config import cfg
+from oslo.utils import timeutils
 
 from heat.common import exception
+from heat.common import short_id
 from heat.common import template_format
-from heat.engine import clients
 from heat.engine import resource
+from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.openstack.common import timeutils
+from heat.engine import stack_resource
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import generic_resource
 from heat.tests import utils
 
@@ -38,7 +40,7 @@ class AutoScalingGroupTest(HeatTestCase):
               max_size: 5
               min_size: 1
               resource:
-                type: ResourceWithProps
+                type: ResourceWithPropsAndAttrs
                 properties:
                     Foo: hello
             type: OS::Heat::AutoScalingGroup
@@ -46,14 +48,11 @@ class AutoScalingGroupTest(HeatTestCase):
 
     def setUp(self):
         super(AutoScalingGroupTest, self).setUp()
-        utils.setup_dummy_db()
-        resource._register_class('ResourceWithProps',
-                                 generic_resource.ResourceWithProps)
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
         cfg.CONF.set_default('heat_waitcondition_server_url',
                              'http://server.test:8000/v1/waitcondition')
-        self.fc = fakes.FakeKeystoneClient()
-        client = self.patchobject(clients.OpenStackClients, "keystone")
-        client.return_value = self.fc
+        self.stub_keystoneclient()
         self.parsed = template_format.parse(self.as_template)
 
     def create_stack(self, t):
@@ -79,8 +78,11 @@ class AutoScalingGroupTest(HeatTestCase):
         self.assertEqual(1, len(resources))
 
         # Reduce the min size to 0, should complete without adjusting
-        update_snippet = copy.deepcopy(rsrc.parsed_template())
-        update_snippet['Properties']['min_size'] = 0
+        props = copy.copy(rsrc.properties.data)
+        props['min_size'] = 0
+        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                                      rsrc.type(),
+                                                      props)
         scheduler.TaskRunner(rsrc.update, update_snippet)()
         self.assertEqual(resources, rsrc.get_instances())
 
@@ -88,15 +90,6 @@ class AutoScalingGroupTest(HeatTestCase):
         rsrc.adjust(-1)
         self.assertEqual(0, len(rsrc.get_instances()))
 
-    def test_scaling_group_update_replace(self):
-        rsrc = self.create_stack(self.parsed)['my-group']
-        self.assertEqual(1, len(rsrc.get_instances()))
-        update_snippet = copy.deepcopy(rsrc.parsed_template())
-        props = update_snippet['Properties']['resource']['properties']
-        props['Foo'] = 'Bar'
-        updater = scheduler.TaskRunner(rsrc.update, update_snippet)
-        self.assertRaises(resource.UpdateReplace, updater)
-
     def test_scaling_group_suspend(self):
         rsrc = self.create_stack(self.parsed)['my-group']
         self.assertEqual(1, len(rsrc.get_instances()))
@@ -138,8 +131,11 @@ class AutoScalingGroupTest(HeatTestCase):
         self.assertEqual(1, len(resources))
 
         # Reduce the max size to 2, should complete without adjusting
-        update_snippet = copy.deepcopy(rsrc.parsed_template())
-        update_snippet['Properties']['max_size'] = 2
+        props = copy.copy(rsrc.properties.data)
+        props['max_size'] = 2
+        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                                      rsrc.type(),
+                                                      props)
         scheduler.TaskRunner(rsrc.update, update_snippet)()
         self.assertEqual(resources, rsrc.get_instances())
         self.assertEqual(2, rsrc.properties['max_size'])
@@ -152,8 +148,11 @@ class AutoScalingGroupTest(HeatTestCase):
         rsrc = self.create_stack(self.parsed)['my-group']
         self.assertEqual(1, len(rsrc.get_instances()))
 
-        update_snippet = copy.deepcopy(rsrc.parsed_template())
-        update_snippet['Properties']['min_size'] = 2
+        props = copy.copy(rsrc.properties.data)
+        props['min_size'] = 2
+        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                                      rsrc.type(),
+                                                      props)
         scheduler.TaskRunner(rsrc.update, update_snippet)()
         self.assertEqual(2, len(rsrc.get_instances()))
         self.assertEqual(2, rsrc.properties['min_size'])
@@ -165,8 +164,11 @@ class AutoScalingGroupTest(HeatTestCase):
         rsrc = self.create_stack(self.parsed)['my-group']
         self.assertEqual(1, len(rsrc.get_instances()))
 
-        update_snippet = copy.deepcopy(rsrc.parsed_template())
-        update_snippet['Properties']['desired_capacity'] = 2
+        props = copy.copy(rsrc.properties.data)
+        props['desired_capacity'] = 2
+        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                                      rsrc.type(),
+                                                      props)
         scheduler.TaskRunner(rsrc.update, update_snippet)()
         self.assertEqual(2, len(rsrc.get_instances()))
         self.assertEqual(2, rsrc.properties['desired_capacity'])
@@ -178,8 +180,11 @@ class AutoScalingGroupTest(HeatTestCase):
         resources = rsrc.get_instances()
         self.assertEqual(2, len(resources))
 
-        update_snippet = copy.deepcopy(rsrc.parsed_template())
-        update_snippet['Properties'].pop('desired_capacity')
+        props = copy.copy(rsrc.properties.data)
+        props.pop('desired_capacity')
+        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                                      rsrc.type(),
+                                                      props)
         scheduler.TaskRunner(rsrc.update, update_snippet)()
         self.assertEqual(resources, rsrc.get_instances())
         self.assertIsNone(rsrc.properties['desired_capacity'])
@@ -248,6 +253,17 @@ class AutoScalingGroupTest(HeatTestCase):
         self.assertRaises(exception.StackValidationFailed,
                           stack['my-group'].validate)
 
+    def test_output_attribute_list(self):
+        rsrc = self.create_stack(self.parsed)['my-group']
+        member_names = rsrc.nested().resources.keys()
+        self.assertEqual(member_names, rsrc.FnGetAtt('outputs_list', 'Bar'))
+
+    def test_output_attribute_dict(self):
+        rsrc = self.create_stack(self.parsed)['my-group']
+        member_names = rsrc.nested().resources.keys()
+        self.assertEqual(dict((n, n) for n in member_names),
+                         rsrc.FnGetAtt('outputs', 'Bar'))
+
 
 class HeatScalingGroupWithCFNScalingPolicyTest(HeatTestCase):
     as_template = '''
@@ -274,14 +290,11 @@ class HeatScalingGroupWithCFNScalingPolicyTest(HeatTestCase):
 
     def setUp(self):
         super(HeatScalingGroupWithCFNScalingPolicyTest, self).setUp()
-        utils.setup_dummy_db()
         resource._register_class('ResourceWithProps',
                                  generic_resource.ResourceWithProps)
         cfg.CONF.set_default('heat_waitcondition_server_url',
                              'http://server.test:8000/v1/waitcondition')
-        self.fc = fakes.FakeKeystoneClient()
-        client = self.patchobject(clients.OpenStackClients, "keystone")
-        client.return_value = self.fc
+        self.stub_keystoneclient()
         self.parsed = template_format.parse(self.as_template)
 
     def create_stack(self, t):
@@ -310,7 +323,7 @@ class HeatScalingGroupWithCFNScalingPolicyTest(HeatTestCase):
 
 
 class ScalingPolicyTest(HeatTestCase):
-
+    # TODO(Qiming): Add more tests to the scaling policy
     as_template = '''
         heat_template_version: 2013-05-23
         resources:
@@ -333,12 +346,9 @@ class ScalingPolicyTest(HeatTestCase):
 
     def setUp(self):
         super(ScalingPolicyTest, self).setUp()
-        utils.setup_dummy_db()
         resource._register_class('ResourceWithProps',
                                  generic_resource.ResourceWithProps)
-        self.fc = fakes.FakeKeystoneClient()
-        client = self.patchobject(clients.OpenStackClients, "keystone")
-        client.return_value = self.fc
+        self.stub_keystoneclient()
         self.parsed = template_format.parse(self.as_template)
 
     def test_alarm_attribute(self):
@@ -376,7 +386,85 @@ class ScalingPolicyTest(HeatTestCase):
 
         past = timeutils.strtime(timeutils.utcnow() -
                                  datetime.timedelta(seconds=65))
-        policy.metadata = {past: 'ChangeInCapacity : 1'}
+        policy.metadata_set({past: 'ChangeInCapacity : 1'})
 
         policy.signal()
         self.assertEqual(3, len(group.get_instance_names()))
+
+
+class RollingUpdatesTest(HeatTestCase):
+
+    as_template = '''
+        heat_template_version: 2013-05-23
+        description: AutoScaling Test
+        resources:
+          my-group:
+            properties:
+              max_size: 5
+              min_size: 4
+              rolling_updates:
+                min_in_service: 2
+                max_batch_size: 2
+                pause_time: 0
+              resource:
+                type: ResourceWithProps
+                properties:
+                    Foo: hello
+            type: OS::Heat::AutoScalingGroup
+    '''
+
+    def setUp(self):
+        super(RollingUpdatesTest, self).setUp()
+        utils.setup_dummy_db()
+        resource._register_class('ResourceWithProps',
+                                 generic_resource.ResourceWithProps)
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+        self.stub_keystoneclient()
+        self.parsed = template_format.parse(self.as_template)
+        generate_id = self.patchobject(short_id, 'generate_id')
+        generate_id.side_effect = ('id-%d' % (i,)
+                                   for i in itertools.count()).next
+
+    def create_stack(self, t):
+        stack = utils.parse_stack(t)
+        stack.create()
+        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
+        return stack
+
+    def test_rolling_update(self):
+        stack = self.create_stack(self.parsed)
+        rsrc = stack['my-group']
+        created_order = sorted(
+            rsrc.nested().resources,
+            key=lambda name: rsrc.nested().resources[name].created_time)
+        batches = []
+
+        def update_with_template(tmpl, env):
+            # keep track of the new updates to resources _in creation order_.
+            definitions = tmpl.resource_definitions(stack)
+            templates = [definitions[name] for name in created_order]
+            batches.append(templates)
+
+        self.patchobject(
+            stack_resource.StackResource, 'update_with_template',
+            side_effect=update_with_template, wraps=rsrc.update_with_template)
+
+        props = copy.deepcopy(rsrc.properties.data)
+        props['resource']['properties']['Foo'] = 'Hi'
+        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name,
+                                                      rsrc.type(),
+                                                      props)
+        scheduler.TaskRunner(rsrc.update, update_snippet)()
+
+        props_schema = generic_resource.ResourceWithProps.properties_schema
+
+        def get_foos(defns):
+            return [d.properties(props_schema)['Foo'] for d in defns]
+
+        # first batch has 2 new resources
+        self.assertEqual(['Hi', 'Hi', 'hello', 'hello'],
+                         get_foos(batches[0]))
+        # second batch has all new resources.
+        self.assertEqual(['Hi', 'Hi', 'Hi', 'Hi'],
+                         get_foos(batches[1]))
