@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,21 +13,22 @@
 
 import copy
 import mox
+from oslo.config import cfg
+import six
 
-from testtools import skipIf
+from neutronclient.common import exceptions
+from neutronclient.neutron import v2_0 as neutronV20
+from neutronclient.v2_0 import client as neutronclient
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
+from heat.engine.clients.os import nova
 from heat.engine.resources.neutron import loadbalancer
 from heat.engine import scheduler
-from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import utils
 from heat.tests.v1_1 import fakes as nova_fakes
 
-neutronclient = try_import('neutronclient.v2_0.client')
 
 health_monitor_template = '''
 {
@@ -58,7 +59,7 @@ pool_template_with_vip_subnet = '''
       "Type": "OS::Neutron::Pool",
       "Properties": {
         "protocol": "HTTP",
-        "subnet_id": "sub123",
+        "subnet": "sub123",
         "lb_method": "ROUND_ROBIN",
         "vip": {
           "protocol_port": 80,
@@ -69,7 +70,8 @@ pool_template_with_vip_subnet = '''
   }
 }
 '''
-pool_template = '''
+
+pool_template_deprecated = '''
 {
   "AWSTemplateFormatVersion" : "2010-09-09",
   "Description" : "Template to test load balancer resources",
@@ -90,6 +92,28 @@ pool_template = '''
 }
 '''
 
+pool_template = '''
+{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+  "Description" : "Template to test load balancer resources",
+  "Parameters" : {},
+  "Resources" : {
+    "pool": {
+      "Type": "OS::Neutron::Pool",
+      "Properties": {
+        "protocol": "HTTP",
+        "subnet": "sub123",
+        "lb_method": "ROUND_ROBIN",
+        "vip": {
+          "protocol_port": 80
+        }
+      }
+    }
+  }
+}
+'''
+
+
 member_template = '''
 {
   "AWSTemplateFormatVersion" : "2010-09-09",
@@ -136,7 +160,7 @@ pool_with_session_persistence_template = '''
       "Type": "OS::Neutron::Pool",
       "Properties": {
         "protocol": "HTTP",
-        "subnet_id": "sub123",
+        "subnet": "sub123",
         "lb_method": "ROUND_ROBIN",
         "vip": {
           "protocol_port": 80,
@@ -196,7 +220,6 @@ pool_with_health_monitors_template = '''
 '''
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class HealthMonitorTest(HeatTestCase):
 
     def setUp(self):
@@ -205,12 +228,9 @@ class HealthMonitorTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client, 'delete_health_monitor')
         self.m.StubOutWithMock(neutronclient.Client, 'show_health_monitor')
         self.m.StubOutWithMock(neutronclient.Client, 'update_health_monitor')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_health_monitor(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_health_monitor({
             'health_monitor': {
                 'delay': 3, 'max_retries': 5, 'type': u'HTTP',
@@ -219,8 +239,9 @@ class HealthMonitorTest(HeatTestCase):
 
         snippet = template_format.parse(health_monitor_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return loadbalancer.HealthMonitor(
-            'monitor', snippet['Resources']['monitor'], stack)
+            'monitor', resource_defns['monitor'], stack)
 
     def test_create(self):
         rsrc = self.create_health_monitor()
@@ -230,31 +251,30 @@ class HealthMonitorTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_health_monitor({
             'health_monitor': {
                 'delay': 3, 'max_retries': 5, 'type': u'HTTP',
                 'timeout': 10, 'admin_state_up': True}}
-        ).AndRaise(loadbalancer.NeutronClientException())
+        ).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(health_monitor_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.HealthMonitor(
-            'monitor', snippet['Resources']['monitor'], stack)
+            'monitor', resource_defns['monitor'], stack)
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete(self):
         neutronclient.Client.delete_health_monitor('5678')
         neutronclient.Client.show_health_monitor('5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_health_monitor()
         self.m.ReplayAll()
@@ -265,7 +285,7 @@ class HealthMonitorTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_health_monitor('5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_health_monitor()
         self.m.ReplayAll()
@@ -276,7 +296,7 @@ class HealthMonitorTest(HeatTestCase):
 
     def test_delete_failed(self):
         neutronclient.Client.delete_health_monitor('5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_health_monitor()
         self.m.ReplayAll()
@@ -285,7 +305,7 @@ class HealthMonitorTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
@@ -308,7 +328,7 @@ class HealthMonitorTest(HeatTestCase):
                                   rsrc.FnGetAtt, 'subnet_id')
         self.assertEqual(
             'The Referenced Attribute (monitor subnet_id) is incorrect.',
-            str(error))
+            six.text_type(error))
         self.m.VerifyAll()
 
     def test_update(self):
@@ -325,7 +345,6 @@ class HealthMonitorTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class PoolTest(HeatTestCase):
 
     def setUp(self):
@@ -339,23 +358,22 @@ class PoolTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client,
                                'disassociate_health_monitor')
         self.m.StubOutWithMock(neutronclient.Client, 'create_vip')
-        self.m.StubOutWithMock(loadbalancer.neutronV20,
-                               'find_resourceid_by_name_or_id')
+        self.m.StubOutWithMock(neutronV20, 'find_resourceid_by_name_or_id')
         self.m.StubOutWithMock(neutronclient.Client, 'delete_vip')
         self.m.StubOutWithMock(neutronclient.Client, 'show_vip')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
-    def create_pool(self, with_vip_subnet=False):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+    def create_pool(self, resolve_neutron=True, with_vip_subnet=False):
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
                 'name': utils.PhysName('test_stack', 'pool'),
                 'lb_method': 'ROUND_ROBIN', 'admin_state_up': True}}
         ).AndReturn({'pool': {'id': '5678'}})
-
+        neutronclient.Client.show_pool('5678').AndReturn(
+            {'pool': {'status': 'ACTIVE'}})
+        neutronclient.Client.show_vip('xyz').AndReturn(
+            {'vip': {'status': 'ACTIVE'}})
         stvipvsn = {
             'vip': {
                 'protocol': u'HTTP', 'name': 'pool.vip',
@@ -366,38 +384,53 @@ class PoolTest(HeatTestCase):
         stvippsn = copy.deepcopy(stvipvsn)
         stvippsn['vip']['subnet_id'] = 'sub123'
 
-        if with_vip_subnet:
+        if resolve_neutron and with_vip_subnet:
+            neutronV20.find_resourceid_by_name_or_id(
+                mox.IsA(neutronclient.Client),
+                'subnet',
+                'sub123'
+            ).AndReturn('sub123')
+            neutronV20.find_resourceid_by_name_or_id(
+                mox.IsA(neutronclient.Client),
+                'subnet',
+                'sub9999'
+            ).AndReturn('sub9999')
+            snippet = template_format.parse(pool_template_with_vip_subnet)
             neutronclient.Client.create_vip(stvipvsn
                                             ).AndReturn({'vip': {'id': 'xyz'}})
-            snippet = template_format.parse(pool_template_with_vip_subnet)
+
+        elif resolve_neutron and not with_vip_subnet:
+            neutronV20.find_resourceid_by_name_or_id(
+                mox.IsA(neutronclient.Client),
+                'subnet',
+                'sub123'
+            ).AndReturn('sub123')
+            snippet = template_format.parse(pool_template)
+            neutronclient.Client.create_vip(stvippsn
+                                            ).AndReturn({'vip': {'id': 'xyz'}})
         else:
+            snippet = template_format.parse(pool_template_deprecated)
             neutronclient.Client.create_vip(stvippsn
                                             ).AndReturn({'vip': {'id': 'xyz'}})
-            snippet = template_format.parse(pool_template)
-
-        neutronclient.Client.show_pool('5678').AndReturn(
-            {'pool': {'status': 'ACTIVE'}})
-        neutronclient.Client.show_vip('xyz').AndReturn(
-            {'vip': {'status': 'ACTIVE'}})
-
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+            'pool', resource_defns['pool'], stack)
 
     def test_create(self):
-        rsrc = self.create_pool()
+        self._test_create()
+
+    def test_create_deprecated(self):
+        self._test_create(resolve_neutron=False, with_vip_subnet=False)
+
+    def _test_create(self, resolve_neutron=True, with_vip_subnet=False):
+        rsrc = self.create_pool(resolve_neutron, with_vip_subnet)
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         self.m.VerifyAll()
 
     def test_create_with_vip_subnet(self):
-        loadbalancer.neutronV20.find_resourceid_by_name_or_id(
-            mox.IsA(neutronclient.Client),
-            'subnet',
-            'sub9999'
-        ).AndReturn('sub9999')
-
         rsrc = self.create_pool(with_vip_subnet=True)
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
@@ -405,8 +438,12 @@ class PoolTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_pending(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
+
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
@@ -430,16 +467,23 @@ class PoolTest(HeatTestCase):
 
         snippet = template_format.parse(pool_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+            'pool', resource_defns['pool'], stack)
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         self.m.VerifyAll()
 
-    def test_create_failed_unexpected_status(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+    def test_create_failed_error_status(self):
+        cfg.CONF.set_override('action_retry_limit', 0)
+
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
+
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
@@ -457,21 +501,25 @@ class PoolTest(HeatTestCase):
 
         snippet = template_format.parse(pool_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+            'pool', resource_defns['pool'], stack)
         self.m.ReplayAll()
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
-            'Error: neutron reported unexpected pool '
-            'resource[5678] status[ERROR]',
-            str(error))
+            'ResourceInError: Went to status ERROR due to "error in pool"',
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_create_failed_unexpected_vip_status(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
+
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
@@ -487,48 +535,57 @@ class PoolTest(HeatTestCase):
         neutronclient.Client.show_pool('5678').MultipleTimes().AndReturn(
             {'pool': {'status': 'ACTIVE'}})
         neutronclient.Client.show_vip('xyz').AndReturn(
-            {'vip': {'status': 'ERROR', 'name': 'xyz'}})
+            {'vip': {'status': 'SOMETHING', 'name': 'xyz'}})
 
         snippet = template_format.parse(pool_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+            'pool', resource_defns['pool'], stack)
         self.m.ReplayAll()
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
-            'Error: neutron reported unexpected vip '
-            'resource[xyz] status[ERROR]',
-            str(error))
+            'ResourceUnknownStatus: Pool creation failed due to vip - '
+            'Unknown status SOMETHING',
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
+
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
                 'name': utils.PhysName('test_stack', 'pool'),
                 'lb_method': 'ROUND_ROBIN', 'admin_state_up': True}}
-        ).AndRaise(loadbalancer.NeutronClientException())
+        ).AndRaise(exceptions.NeutronClientException())
         self.m.ReplayAll()
 
         snippet = template_format.parse(pool_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+            'pool', resource_defns['pool'], stack)
         error = self.assertRaises(exception.ResourceFailure,
                                   scheduler.TaskRunner(rsrc.create))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_create_with_session_persistence(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
@@ -551,8 +608,9 @@ class PoolTest(HeatTestCase):
 
         snippet = template_format.parse(pool_with_session_persistence_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+            'pool', resource_defns['pool'], stack)
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
@@ -569,23 +627,30 @@ class PoolTest(HeatTestCase):
         persistence['type'] = 'APP_COOKIE'
         persistence['cookie_name'] = None
 
-        resource = loadbalancer.Pool('pool', pool, utils.parse_stack(snippet))
+        stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
+        resource = loadbalancer.Pool('pool', resource_defns['pool'], stack)
 
         error = self.assertRaises(exception.StackValidationFailed,
                                   resource.validate)
-        self.assertEqual(msg, str(error))
+        self.assertEqual(msg, six.text_type(error))
 
     def test_validation_not_failing_without_session_persistence(self):
         snippet = template_format.parse(pool_template)
-        pool = snippet['Resources']['pool']
 
-        resource = loadbalancer.Pool('pool', pool, utils.parse_stack(snippet))
+        stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
+        resource = loadbalancer.Pool('pool', resource_defns['pool'], stack)
 
         self.assertIsNone(resource.validate())
 
     def test_properties_are_prepared_for_session_persistence(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
+
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
@@ -611,7 +676,10 @@ class PoolTest(HeatTestCase):
         # change persistence type to HTTP_COOKIE that not require cookie_name
         persistence['type'] = 'HTTP_COOKIE'
         del persistence['cookie_name']
-        resource = loadbalancer.Pool('pool', pool, utils.parse_stack(snippet))
+
+        stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
+        resource = loadbalancer.Pool('pool', resource_defns['pool'], stack)
 
         # assert that properties contain cookie_name property with None value
         persistence = resource.properties['vip']['session_persistence']
@@ -627,10 +695,10 @@ class PoolTest(HeatTestCase):
         rsrc = self.create_pool()
         neutronclient.Client.delete_vip('xyz')
         neutronclient.Client.show_vip('xyz').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
         neutronclient.Client.delete_pool('5678')
         neutronclient.Client.show_pool('5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         scheduler.TaskRunner(rsrc.delete)()
@@ -639,9 +707,9 @@ class PoolTest(HeatTestCase):
 
     def test_delete_already_gone(self):
         neutronclient.Client.delete_vip('xyz').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
         neutronclient.Client.delete_pool('5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         rsrc = self.create_pool()
         self.m.ReplayAll()
@@ -652,7 +720,7 @@ class PoolTest(HeatTestCase):
 
     def test_delete_vip_failed(self):
         neutronclient.Client.delete_vip('xyz').AndRaise(
-            loadbalancer.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_pool()
         self.m.ReplayAll()
@@ -661,15 +729,15 @@ class PoolTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
     def test_delete_failed(self):
         neutronclient.Client.delete_vip('xyz').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
         neutronclient.Client.delete_pool('5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=400))
+            exceptions.NeutronClientException(status_code=400))
 
         rsrc = self.create_pool()
         self.m.ReplayAll()
@@ -678,7 +746,7 @@ class PoolTest(HeatTestCase):
                                   scheduler.TaskRunner(rsrc.delete))
         self.assertEqual(
             'NeutronClientException: An unknown exception occurred.',
-            str(error))
+            six.text_type(error))
         self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
 
@@ -711,7 +779,7 @@ class PoolTest(HeatTestCase):
                                   rsrc.FnGetAtt, 'net_id')
         self.assertEqual(
             'The Referenced Attribute (pool net_id) is incorrect.',
-            str(error))
+            six.text_type(error))
         self.m.VerifyAll()
 
     def test_update(self):
@@ -728,8 +796,11 @@ class PoolTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_update_monitors(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123'
+        ).AndReturn('sub123')
         neutronclient.Client.create_pool({
             'pool': {
                 'subnet_id': 'sub123', 'protocol': u'HTTP',
@@ -759,8 +830,8 @@ class PoolTest(HeatTestCase):
         stack = utils.parse_stack(snippet)
         snippet['Resources']['pool']['Properties']['monitors'] = [
             'mon123', 'mon456']
-        rsrc = loadbalancer.Pool(
-            'pool', snippet['Resources']['pool'], stack)
+        resource_defns = stack.t.resource_definitions(stack)
+        rsrc = loadbalancer.Pool('pool', resource_defns['pool'], stack)
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
 
@@ -771,7 +842,6 @@ class PoolTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class PoolMemberTest(HeatTestCase):
 
     def setUp(self):
@@ -781,12 +851,9 @@ class PoolMemberTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client, 'delete_member')
         self.m.StubOutWithMock(neutronclient.Client, 'update_member')
         self.m.StubOutWithMock(neutronclient.Client, 'show_member')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
     def create_member(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_member({
             'member': {
                 'pool_id': 'pool123', 'protocol_port': 8080,
@@ -794,8 +861,9 @@ class PoolMemberTest(HeatTestCase):
         ).AndReturn({'member': {'id': 'member5678'}})
         snippet = template_format.parse(member_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return loadbalancer.PoolMember(
-            'member', snippet['Resources']['member'], stack)
+            'member', resource_defns['member'], stack)
 
     def test_create(self):
         rsrc = self.create_member()
@@ -807,8 +875,6 @@ class PoolMemberTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_optional_parameters(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         neutronclient.Client.create_member({
             'member': {
                 'pool_id': 'pool123', 'protocol_port': 8080,
@@ -819,8 +885,9 @@ class PoolMemberTest(HeatTestCase):
         snippet['Resources']['member']['Properties']['admin_state_up'] = False
         snippet['Resources']['member']['Properties']['weight'] = 100
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = loadbalancer.PoolMember(
-            'member', snippet['Resources']['member'], stack)
+            'member', resource_defns['member'], stack)
 
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
@@ -857,7 +924,7 @@ class PoolMemberTest(HeatTestCase):
         rsrc = self.create_member()
         neutronclient.Client.delete_member(u'member5678')
         neutronclient.Client.show_member(u'member5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
@@ -868,7 +935,7 @@ class PoolMemberTest(HeatTestCase):
     def test_delete_missing_member(self):
         rsrc = self.create_member()
         neutronclient.Client.delete_member(u'member5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
@@ -877,7 +944,6 @@ class PoolMemberTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class LoadBalancerTest(HeatTestCase):
 
     def setUp(self):
@@ -885,15 +951,11 @@ class LoadBalancerTest(HeatTestCase):
         self.fc = nova_fakes.FakeClient()
         self.m.StubOutWithMock(neutronclient.Client, 'create_member')
         self.m.StubOutWithMock(neutronclient.Client, 'delete_member')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
 
     def create_load_balancer(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
-        clients.OpenStackClients.nova("compute").MultipleTimes().AndReturn(
-            self.fc)
+        nova.NovaClientPlugin._create().AndReturn(self.fc)
         neutronclient.Client.create_member({
             'member': {
                 'pool_id': 'pool123', 'protocol_port': 8080,
@@ -901,8 +963,9 @@ class LoadBalancerTest(HeatTestCase):
         ).AndReturn({'member': {'id': 'member5678'}})
         snippet = template_format.parse(lb_template)
         stack = utils.parse_stack(snippet)
+        resource_defns = stack.t.resource_definitions(stack)
         return loadbalancer.LoadBalancer(
-            'lb', snippet['Resources']['lb'], stack)
+            'lb', resource_defns['lb'], stack)
 
     def test_create(self):
         rsrc = self.create_load_balancer()
@@ -933,7 +996,7 @@ class LoadBalancerTest(HeatTestCase):
     def test_update_missing_member(self):
         rsrc = self.create_load_balancer()
         neutronclient.Client.delete_member(u'member5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
@@ -958,7 +1021,7 @@ class LoadBalancerTest(HeatTestCase):
     def test_delete_missing_member(self):
         rsrc = self.create_load_balancer()
         neutronclient.Client.delete_member(u'member5678').AndRaise(
-            loadbalancer.NeutronClientException(status_code=404))
+            exceptions.NeutronClientException(status_code=404))
 
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
@@ -967,7 +1030,6 @@ class LoadBalancerTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-@skipIf(neutronclient is None, 'neutronclient unavailable')
 class PoolUpdateHealthMonitorsTest(HeatTestCase):
 
     def setUp(self):
@@ -987,13 +1049,9 @@ class PoolUpdateHealthMonitorsTest(HeatTestCase):
         self.m.StubOutWithMock(neutronclient.Client, 'create_vip')
         self.m.StubOutWithMock(neutronclient.Client, 'delete_vip')
         self.m.StubOutWithMock(neutronclient.Client, 'show_vip')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-        utils.setup_dummy_db()
+        self.stub_keystoneclient()
 
-    @utils.stack_delete_after
-    def test_update_pool_with_references_to_health_monitors(self):
-        clients.OpenStackClients.keystone().MultipleTimes().AndReturn(
-            fakes.FakeKeystoneClient())
+    def _create_pool_with_health_monitors(self):
         neutronclient.Client.create_health_monitor({
             'health_monitor': {
                 'delay': 3, 'max_retries': 5, 'type': u'HTTP',
@@ -1026,6 +1084,9 @@ class PoolUpdateHealthMonitorsTest(HeatTestCase):
         neutronclient.Client.show_vip('xyz').AndReturn(
             {'vip': {'status': 'ACTIVE'}})
 
+    def test_update_pool_with_references_to_health_monitors(self):
+        self._create_pool_with_health_monitors()
+
         neutronclient.Client.disassociate_health_monitor(
             '5678', mox.IsA(unicode))
 
@@ -1043,3 +1104,47 @@ class PoolUpdateHealthMonitorsTest(HeatTestCase):
         self.assertEqual((self.stack.UPDATE, self.stack.COMPLETE),
                          self.stack.state)
         self.m.VerifyAll()
+
+    def test_update_pool_with_empty_list_of_health_monitors(self):
+        self._create_pool_with_health_monitors()
+
+        neutronclient.Client.disassociate_health_monitor(
+            '5678', '5555').InAnyOrder()
+        neutronclient.Client.disassociate_health_monitor(
+            '5678', '6666').InAnyOrder()
+
+        self.m.ReplayAll()
+        snippet = template_format.parse(pool_with_health_monitors_template)
+        self.stack = utils.parse_stack(snippet)
+        self.stack.create()
+        self.assertEqual((self.stack.CREATE, self.stack.COMPLETE),
+                         self.stack.state)
+
+        snippet['Resources']['pool']['Properties']['monitors'] = []
+        updated_stack = utils.parse_stack(snippet)
+        self.stack.update(updated_stack)
+        self.assertEqual((self.stack.UPDATE, self.stack.COMPLETE),
+                         self.stack.state)
+        self.m.VerifyAll()
+
+    def test_update_pool_without_health_monitors(self):
+        self._create_pool_with_health_monitors()
+
+        neutronclient.Client.disassociate_health_monitor(
+            '5678', '5555').InAnyOrder()
+        neutronclient.Client.disassociate_health_monitor(
+            '5678', '6666').InAnyOrder()
+
+        self.m.ReplayAll()
+        snippet = template_format.parse(pool_with_health_monitors_template)
+        self.stack = utils.parse_stack(snippet)
+        self.stack.create()
+        self.assertEqual((self.stack.CREATE, self.stack.COMPLETE),
+                         self.stack.state)
+
+        snippet['Resources']['pool']['Properties'].pop('monitors')
+        updated_stack = utils.parse_stack(snippet)
+        self.stack.update(updated_stack)
+        self.assertEqual((self.stack.UPDATE, self.stack.COMPLETE),
+                         self.stack.state)
+        self.m.VerifyAll()
