@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -18,11 +18,10 @@ from novaclient.v1_1 import security_groups as nova_sg
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
 from heat.engine import parser
 from heat.engine import scheduler
+from heat.engine import template
 from heat.tests.common import HeatTestCase
-from heat.tests.fakes import FakeKeystoneClient
 from heat.tests import utils
 from heat.tests.v1_1 import fakes
 
@@ -87,15 +86,13 @@ Resources:
     def setUp(self):
         super(SecurityGroupTest, self).setUp()
         self.fc = fakes.FakeClient()
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
+        self.stub_keystoneclient()
         self.m.StubOutWithMock(nova_sgr.SecurityGroupRuleManager, 'create')
         self.m.StubOutWithMock(nova_sgr.SecurityGroupRuleManager, 'delete')
         self.m.StubOutWithMock(nova_sg.SecurityGroupManager, 'create')
         self.m.StubOutWithMock(nova_sg.SecurityGroupManager, 'delete')
         self.m.StubOutWithMock(nova_sg.SecurityGroupManager, 'get')
         self.m.StubOutWithMock(nova_sg.SecurityGroupManager, 'list')
-        utils.setup_dummy_db()
         self.m.StubOutWithMock(neutronclient.Client, 'create_security_group')
         self.m.StubOutWithMock(
             neutronclient.Client, 'create_security_group_rule')
@@ -105,26 +102,26 @@ Resources:
         self.m.StubOutWithMock(neutronclient.Client, 'delete_security_group')
         self.m.StubOutWithMock(neutronclient.Client, 'update_security_group')
 
-    def create_stack(self, template):
-        t = template_format.parse(template)
+    def create_stack(self, templ):
+        t = template_format.parse(templ)
         self.stack = self.parse_stack(t)
         self.assertIsNone(self.stack.create())
         return self.stack
 
     def parse_stack(self, t):
         stack_name = 'test_stack'
-        tmpl = parser.Template(t)
+        tmpl = template.Template(t)
         stack = parser.Stack(utils.dummy_context(), stack_name, tmpl)
         stack.store()
         return stack
 
-    def assertResourceState(self, rsrc, ref_id, metadata={}):
+    def assertResourceState(self, rsrc, ref_id, metadata=None):
+        metadata = metadata or {}
         self.assertIsNone(rsrc.validate())
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         self.assertEqual(ref_id, rsrc.FnGetRefId())
-        self.assertEqual(metadata, dict(rsrc.metadata))
+        self.assertEqual(metadata, dict(rsrc.metadata_get()))
 
-    @utils.stack_delete_after
     def test_security_group(self):
 
         show_created = {'security_group': {
@@ -202,8 +199,6 @@ Resources:
         }
 
         #create script
-        clients.OpenStackClients.keystone().AndReturn(
-            FakeKeystoneClient())
         sg_name = utils.PhysName('test_stack', 'the_sg')
         neutronclient.Client.create_security_group({
             'security_group': {
@@ -544,11 +539,8 @@ Resources:
         stack.delete()
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_security_group_exception(self):
         #create script
-        clients.OpenStackClients.keystone().AndReturn(
-            FakeKeystoneClient())
         sg_name = utils.PhysName('test_stack', 'the_sg')
         neutronclient.Client.create_security_group({
             'security_group': {
@@ -760,7 +752,6 @@ Resources:
 
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_security_group_validate(self):
         stack = self.create_stack(self.test_template_validate)
         sg = stack['the_sg']
