@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -17,11 +17,11 @@ import uuid
 import mox
 
 from heat.common import template_format
-from heat.engine import clients
+from heat.engine.clients.os import glance
+from heat.engine.clients.os import nova
 from heat.engine import environment
 from heat.engine import parser
 from heat.engine.resources import instance as instances
-from heat.engine.resources import nova_utils
 from heat.engine import scheduler
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
@@ -128,7 +128,11 @@ class ServerTagsTest(HeatTestCase):
     def setUp(self):
         super(ServerTagsTest, self).setUp()
         self.fc = fakes.FakeClient()
-        utils.setup_dummy_db()
+
+    def _mock_get_image_id_success(self, imageId_input, imageId):
+        self.m.StubOutWithMock(glance.GlanceClientPlugin, 'get_image_id')
+        glance.GlanceClientPlugin.get_image_id(
+            imageId_input).MultipleTimes().AndReturn(imageId)
 
     def _setup_test_instance(self, intags=None, nova_tags=None):
         stack_name = 'tag_test'
@@ -139,24 +143,22 @@ class ServerTagsTest(HeatTestCase):
                              stack_id=str(uuid.uuid4()))
 
         t['Resources']['WebServer']['Properties']['Tags'] = intags
+        resource_defns = template.resource_definitions(stack)
         instance = instances.Instance(stack_name,
-                                      t['Resources']['WebServer'], stack)
-
-        self.m.StubOutWithMock(instance, 'nova')
-        instance.nova().MultipleTimes().AndReturn(self.fc)
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
-        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
-
-        instance.t = instance.stack.resolve_runtime_data(instance.t)
+                                      resource_defns['WebServer'], stack)
 
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        nova.NovaClientPlugin._create().AndReturn(self.fc)
+        self._mock_get_image_id_success('CentOS 5.2', 1)
         # need to resolve the template functions
-        server_userdata = nova_utils.build_userdata(
-            instance,
+        metadata = instance.metadata_get()
+        server_userdata = instance.client_plugin().build_userdata(
+            metadata,
             instance.t['Properties']['UserData'],
             'ec2-user')
-        self.m.StubOutWithMock(nova_utils, 'build_userdata')
-        nova_utils.build_userdata(
-            instance,
+        self.m.StubOutWithMock(nova.NovaClientPlugin, 'build_userdata')
+        nova.NovaClientPlugin.build_userdata(
+            metadata,
             instance.t['Properties']['UserData'],
             'ec2-user').AndReturn(server_userdata)
 
@@ -166,7 +168,8 @@ class ServerTagsTest(HeatTestCase):
             name=utils.PhysName(stack_name, instance.name),
             security_groups=None,
             userdata=server_userdata, scheduler_hints=None,
-            meta=nova_tags, nics=None, availability_zone=None).AndReturn(
+            meta=nova_tags, nics=None, availability_zone=None,
+            block_device_mapping=None).AndReturn(
                 self.fc.servers.list()[1])
 
         return instance
@@ -198,13 +201,11 @@ class ServerTagsTest(HeatTestCase):
         new_tags = [{'Key': 'Food', 'Value': 'yuk'}]
         new_metadata = dict((tm['Key'], tm['Value']) for tm in new_tags)
 
-        self.m.StubOutWithMock(instance, 'nova')
-        instance.nova().MultipleTimes().AndReturn(self.fc)
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
-        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
         self.m.StubOutWithMock(self.fc.servers, 'set_meta')
         self.fc.servers.set_meta(self.fc.servers.list()[1],
                                  new_metadata).AndReturn(None)
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
         self.m.ReplayAll()
         update_template = copy.deepcopy(instance.t)
         update_template['Properties']['Tags'] = new_tags
@@ -224,6 +225,9 @@ class ServerTagsTest(HeatTestCase):
 
         # create the launch configuration
         conf = stack['Config']
+        self.stub_KeypairConstraint_validate()
+        self.stub_ImageConstraint_validate()
+        self.m.ReplayAll()
         self.assertIsNone(conf.validate())
         scheduler.TaskRunner(conf.create)()
         self.assertEqual((conf.CREATE, conf.COMPLETE), conf.state)
@@ -232,14 +236,12 @@ class ServerTagsTest(HeatTestCase):
 
         nova_tags['metering.groupname'] = utils.PhysName(stack.name,
                                                          group.name)
+        self.m.VerifyAll()
+        self.m.UnsetStubs()
 
-        self.m.StubOutWithMock(instances.Instance, 'nova')
-        instances.Instance.nova().MultipleTimes().AndReturn(self.fc)
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
-        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
-
-        group.t = group.stack.resolve_runtime_data(group.t)
-
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        nova.NovaClientPlugin._create().AndReturn(self.fc)
+        self._mock_get_image_id_success('CentOS 5.2', 1)
         # need to resolve the template functions
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
@@ -247,7 +249,8 @@ class ServerTagsTest(HeatTestCase):
             name=mox.IgnoreArg(),
             security_groups=None,
             userdata=mox.IgnoreArg(), scheduler_hints=None,
-            meta=nova_tags, nics=None, availability_zone=None).AndReturn(
+            meta=nova_tags, nics=None, availability_zone=None,
+            block_device_mapping=None).AndReturn(
                 self.fc.servers.list()[1])
 
         return group
@@ -273,6 +276,10 @@ class ServerTagsTest(HeatTestCase):
 
         # create the launch configuration
         conf = stack['Config']
+        self.stub_ImageConstraint_validate()
+        self.stub_KeypairConstraint_validate()
+        self.m.ReplayAll()
+
         self.assertIsNone(conf.validate())
         scheduler.TaskRunner(conf.create)()
         self.assertEqual((conf.CREATE, conf.COMPLETE), conf.state)
@@ -281,18 +288,17 @@ class ServerTagsTest(HeatTestCase):
         group_refid = utils.PhysName(stack.name, group.name)
 
         nova_tags['metering.groupname'] = group_refid
-        nova_tags['AutoScalingGroupName'] = group_refid
+        nova_tags['metering.AutoScalingGroupName'] = group_refid
+
+        self.m.VerifyAll()
+        self.m.UnsetStubs()
 
         self.m.StubOutWithMock(group, '_cooldown_timestamp')
         group._cooldown_timestamp(mox.IgnoreArg()).AndReturn(None)
 
-        self.m.StubOutWithMock(instances.Instance, 'nova')
-        instances.Instance.nova().MultipleTimes().AndReturn(self.fc)
-        self.m.StubOutWithMock(clients.OpenStackClients, 'nova')
-        clients.OpenStackClients.nova().MultipleTimes().AndReturn(self.fc)
-
-        group.t = group.stack.resolve_runtime_data(group.t)
-
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        nova.NovaClientPlugin._create().AndReturn(self.fc)
+        self._mock_get_image_id_success('CentOS 5.2', 1)
         # need to resolve the template functions
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
@@ -300,7 +306,8 @@ class ServerTagsTest(HeatTestCase):
             name=mox.IgnoreArg(),
             security_groups=None,
             userdata=mox.IgnoreArg(), scheduler_hints=None,
-            meta=nova_tags, nics=None, availability_zone=None).AndReturn(
+            meta=nova_tags, nics=None, availability_zone=None,
+            block_device_mapping=None).AndReturn(
                 self.fc.servers.list()[1])
 
         return group
