@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -14,13 +14,12 @@
 import os
 
 import mock
-import testtools
+import six
 import yaml
 
 from heat.common import config
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
@@ -83,7 +82,7 @@ class YamlMinimalTest(HeatTestCase):
         parse_ex = self.assertRaises(ValueError,
                                      template_format.parse,
                                      tmpl_str)
-        self.assertIn(msg_str, str(parse_ex))
+        self.assertIn(msg_str, six.text_type(parse_ex))
 
     def test_long_yaml(self):
         template = {'HeatTemplateFormatVersion': '2012-12-12'}
@@ -96,7 +95,7 @@ class YamlMinimalTest(HeatTestCase):
                                template_format.parse, long_yaml)
         msg = ('Request limit exceeded: Template exceeds maximum allowed size '
                '(1024 bytes)')
-        self.assertEqual(msg, str(ex))
+        self.assertEqual(msg, six.text_type(ex))
 
     def test_parse_no_version_format(self):
         yaml = ''
@@ -159,7 +158,6 @@ class JsonYamlResolvedCompareTest(HeatTestCase):
         super(JsonYamlResolvedCompareTest, self).setUp()
         self.longMessage = True
         self.maxDiff = None
-        utils.setup_dummy_db()
 
     def load_template(self, file_name):
         filepath = os.path.join(os.path.dirname(os.path.realpath(__file__)),
@@ -172,8 +170,8 @@ class JsonYamlResolvedCompareTest(HeatTestCase):
     def compare_stacks(self, json_file, yaml_file, parameters):
         t1 = self.load_template(json_file)
         t2 = self.load_template(yaml_file)
-        del(t2[u'HeatTemplateFormatVersion'])
         del(t1[u'AWSTemplateFormatVersion'])
+        t1[u'HeatTemplateFormatVersion'] = t2[u'HeatTemplateFormatVersion']
         stack1 = utils.parse_stack(t1, parameters)
         stack2 = utils.parse_stack(t2, parameters)
 
@@ -190,8 +188,6 @@ class JsonYamlResolvedCompareTest(HeatTestCase):
         for key in stack1:
             self.assertEqual(stack1[key].t, stack2[key].t)
 
-    @testtools.skipIf(clients.neutronclient is None,
-                      'neutronclient unavailable')
     def test_neutron_resolved(self):
         self.compare_stacks('Neutron.template', 'Neutron.yaml', {})
 
