@@ -16,20 +16,20 @@
 """Sample plugin for Heat.
 
 For more Heat related benchmarks take a look here:
-https://git.openstack.org/cgit/openstack/rally/tree/rally/plugins/openstack/scenarios/heat
+http://github.com/stackforge/rally/blob/master/rally/benchmark/scenarios/heat/
 
-About plugins: https://rally.readthedocs.io/en/latest/plugins/#rally-plugins
+About plugins: https://rally.readthedocs.org/en/latest/plugins.html
 
 Rally concepts https://wiki.openstack.org/wiki/Rally/Concepts
 """
 
 
-from rally.plugins.openstack import scenario
+from rally.benchmark.scenarios import base
 
 
-class HeatPlugin(scenario.OpenStackScenario):
+class HeatPlugin(base.Scenario):
 
-    @scenario.configure(context={"cleanup": ["heat"]})
+    @base.scenario(context={"cleanup": ["heat"]})
     def list_benchmark(self, container_format,
                             image_location, disk_format, **kwargs):
         """Get heatclient and do whatever."""
